try:
    import aiohttp
except ImportError:
    aiohttp = None
import unittest
from io import BytesIO
from slixmpp.test.integration import SlixIntegration


class TestHTTPUpload(SlixIntegration):
    async def asyncSetUp(self):
        await super().asyncSetUp()
        self.add_client(
            self.envjid('CI_ACCOUNT1'),
            self.envstr('CI_ACCOUNT1_PASSWORD'),
        )
        self.register_plugins(['xep_0363'])
        # Minimal data, we do not want to clutter the remote server
        self.data = b'tototo'
        await self.connect_clients()


    @unittest.skipIf(aiohttp is None, "aiohttp is not installed")
    async def test_httpupload(self):
        """Check we can upload a file properly."""
        url = await self.clients[0]['xep_0363'].upload_file(
            'toto.txt',
            input_file=BytesIO(self.data),
            size=len(self.data),
        )
        async with aiohttp.ClientSession() as session:
            async with session.get(url) as resp:
                text = await resp.text()
        self.assertEqual(text.encode('utf-8'), self.data)


suite = unittest.TestLoader().loadTestsFromTestCase(TestHTTPUpload)



#!/usr/bin/env python3

# Slixmpp: The Slick XMPP Library
# Copyright (C) 2021 Mathieu Pasquet
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.

import logging
from getpass import getpass
from argparse import ArgumentParser

import asyncio
import slixmpp


class MIXBot(slixmpp.ClientXMPP):

    """
    A simple Slixmpp bot that will greets those
    who enter the room, and acknowledge any messages
    that mentions the bot's nickname.
    """

    def __init__(self, jid, password, room, nick):
        slixmpp.ClientXMPP.__init__(self, jid, password)

        self.room = room
        self.rooms = set()
        self.nick = nick

        # The session_start event will be triggered when
        # the bot establishes its connection with the server
        # and the XML streams are ready for use. We want to
        # listen for this event so that we we can initialize
        # our roster.
        self.add_event_handler("session_start", self.start)

        # The mix_message event is triggered whenever a message
        # stanza is received from any chat room.
        self.add_event_handler("mix_message", self.mix_message)

        # The mix_participant_info_publish event is triggered whenever
        # an occupant joins or leaves the channel (not linked to
        # actual presence)
        self.add_event_handler("mix_participant_info_publish",
                               self.mix_joined)


    async def start(self, event):
        """
        Process the session_start event.

        Typical actions for the session_start event are
        requesting the roster and broadcasting an initial
        presence stanza.
        """
        # The goal here is to fetch the already joined MIX channels
        # which are present in the roster.
        _, mix_rooms = await self.plugin['xep_0405'].get_mix_roster()
        for room in mix_rooms:
            self.rooms.add(room['jid'])
        self.send_presence()
        if self.room not in self.rooms:
            # If we are not joined, we need to. This will carry over
            # the next restarts
            await self.plugin['xep_0405'].join_channel(
                self.room,
                self.nick,
            )

    def mix_message(self, msg):
        """
        Process incoming message stanzas from any chat room. Be aware
        that if you also have any handlers for the 'message' event,
        message stanzas may be processed by both handlers, so check
        the 'type' attribute when using a 'message' event handler.

        Whenever the bot's nickname is mentioned, respond to
        the message.

        IMPORTANT: Always check that a message is not from yourself,
                   otherwise you will create an infinite loop responding
                   to your own messages.

        This handler will reply to messages that mention
        the bot's nickname.

        Arguments:
            msg -- The received message stanza. See the documentation
                   for stanza objects and the Message stanza to see
                   how it may be used.
        """
        if msg['mix']['nick'] != self.nick and self.nick in msg['body']:
            self.send_message(mto=msg['from'].bare,
                              mbody="I heard that, %s." % msg['mix']['nick'],
                              mtype='groupchat')

    def mix_joined(self, event):
        """
        We receive a publish event whenever someone joins the MIX channel.
        It contains the nickname of the new participant, and the JID when
        the channel is not a "JID Hidden channel".
        """
        participant = event['pubsub_event']['items']['item']['mix_participant']
        if participant['nick'] != self.nick:
            self.send_message(mto=event['from'].bare,
                              mbody="Hello, %s" % participant['nick'],
                              mtype='groupchat')


if __name__ == '__main__':
    # Setup the command line arguments.
    parser = ArgumentParser()

    # Output verbosity options.
    parser.add_argument("-q", "--quiet", help="set logging to ERROR",
                        action="store_const", dest="loglevel",
                        const=logging.ERROR, default=logging.INFO)
    parser.add_argument("-d", "--debug", help="set logging to DEBUG",
                        action="store_const", dest="loglevel",
                        const=logging.DEBUG, default=logging.INFO)

    # JID and password options.
    parser.add_argument("-j", "--jid", dest="jid",
                        help="JID to use")
    parser.add_argument("-p", "--password", dest="password",
                        help="password to use")
    parser.add_argument("-r", "--room", dest="room",
                        help="MIX channel to join")
    parser.add_argument("-n", "--nick", dest="nick",
                        help="MIX nickname")

    args = parser.parse_args()

    # Setup logging.
    logging.basicConfig(level=args.loglevel,
                        format='%(levelname)-8s %(message)s')

    if args.jid is None:
        args.jid = input("Username: ")
    if args.password is None:
        args.password = getpass("Password: ")
    if args.room is None:
        args.room = input("MIX channel: ")
    if args.nick is None:
        args.nick = input("MIX nickname: ")

    # Setup the MIXBot and register plugins. Note that while plugins may
    # have interdependencies, the order in which you register them does
    # not matter.
    xmpp = MIXBot(args.jid, args.password, args.room, args.nick)
    xmpp.register_plugin('xep_0030') # Service Discovery
    xmpp.register_plugin('xep_0199') # XMPP Ping
    xmpp.register_plugin('xep_0369') # MIX Core
    xmpp.register_plugin('xep_0405') # MIX PAM

    # Connect to the XMPP server and start processing XMPP stanzas.
    xmpp.connect()
    asyncio.get_event_loop().run_forever()




# slixmpp.xmlstream.handler.base
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Part of Slixmpp: The Slick XMPP Library
# :copyright: (c) 2011 Nathanael C. Fritz
# :license: MIT, see LICENSE for more details
from __future__ import annotations

import weakref
from weakref import ReferenceType
from typing import Optional, TYPE_CHECKING, Union
from slixmpp.xmlstream.matcher.base import MatcherBase
from xml.etree.ElementTree import Element

if TYPE_CHECKING:
    from slixmpp.xmlstream import XMLStream, StanzaBase


class BaseHandler:

    """
    Base class for stream handlers. Stream handlers are matched with
    incoming stanzas so that the stanza may be processed in some way.
    Stanzas may be matched with multiple handlers.

    Handler execution may take place in two phases: during the incoming
    stream processing, and in the main event loop. The :meth:`prerun()`
    method is executed in the first case, and :meth:`run()` is called
    during the second.

    :param string name: The name of the handler.
    :param matcher: A :class:`~slixmpp.xmlstream.matcher.base.MatcherBase`
                    derived object that will be used to determine if a
                    stanza should be accepted by this handler.
    :param stream: The :class:`~slixmpp.xmlstream.xmlstream.XMLStream`
                    instance that the handle will respond to.
    """
    name: str
    stream: Optional[ReferenceType[XMLStream]]
    _destroy: bool
    _matcher: MatcherBase
    _payload: Optional[StanzaBase]

    def __init__(self, name: str, matcher: MatcherBase, stream: Optional[XMLStream] = None):
        #: The name of the handler
        self.name = name

        #: The XML stream this handler is assigned to
        self.stream = None
        if stream is not None:
            self.stream = weakref.ref(stream)
            stream.register_handler(self)

        self._destroy = False
        self._payload = None
        self._matcher = matcher

    def match(self, xml: StanzaBase) -> bool:
        """Compare a stanza or XML object with the handler's matcher.

        :param xml: An XML or
            :class:`~slixmpp.xmlstream.stanzabase.StanzaBase` object
        """
        return self._matcher.match(xml)

    def prerun(self, payload: StanzaBase) -> None:
        """Prepare the handler for execution while the XML
        stream is being processed.

        :param payload: A :class:`~slixmpp.xmlstream.stanzabase.StanzaBase`
                        object.
        """
        self._payload = payload

    def run(self, payload: StanzaBase) -> None:
        """Execute the handler after XML stream processing and during the
        main event loop.

        :param payload: A :class:`~slixmpp.xmlstream.stanzabase.StanzaBase`
                        object.
        """
        self._payload = payload

    def check_delete(self) -> bool:
        """Check if the handler should be removed from the list
        of stream handlers.
        """
        return self._destroy




# Slixmpp: The Slick XMPP Library
# Copyright (C) 2020 Mathieu Pasquet
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.
import logging

import slixmpp
from slixmpp import Message
from slixmpp.plugins import BasePlugin
from slixmpp.xmlstream import register_stanza_plugin
from slixmpp.xmlstream.handler import Callback
from slixmpp.xmlstream.matcher import StanzaPath
from slixmpp.plugins.xep_0377 import stanza
from slixmpp.plugins.xep_0191 import Block


log = logging.getLogger(__name__)


class XEP_0377(BasePlugin):
    """XEP-0377: Spam reporting"""

    name = 'xep_0377'
    description = 'XEP-0377: Spam Reporting'
    dependencies = {'xep_0030', 'xep_0191'}
    stanza = stanza

    SPAM = 'urn:xmpp:reporting:spam'
    ABUSE = 'urn:xmpp:reporting:abuse'

    def plugin_init(self):
        register_stanza_plugin(Block, stanza.Report)
        register_stanza_plugin(stanza.Report, stanza.Text)

    def plugin_end(self):
        self.xmpp['xep_0030'].del_feature(feature=stanza.Report.namespace)

    def session_bind(self, jid):
        self.xmpp['xep_0030'].add_feature(stanza.Report.namespace)





# Slixmpp: The Slick XMPP Library
# Copyright (C) 2012 Nathanael C. Fritz, Lance J.T. Stout
# This file is part of Slixmpp.
# See the file LICENSE for copying permissio
from slixmpp.plugins.base import register_plugin

from slixmpp.plugins.xep_0071.stanza import XHTML_IM
from slixmpp.plugins.xep_0071.xhtml_im import XEP_0071


register_plugin(XEP_0071)



import unittest

from slixmpp import Message
from slixmpp.test import SlixTest
import slixmpp.plugins.xep_0004 as xep_0004
import slixmpp.plugins.xep_0122 as xep_0122
from slixmpp.xmlstream import register_stanza_plugin


class TestDataForms(SlixTest):

    def setUp(self):
        register_stanza_plugin(Message, xep_0004.Form)
        register_stanza_plugin(xep_0004.Form, xep_0004.FormField, iterable=True)
        register_stanza_plugin(xep_0004.FormField, xep_0004.FieldOption, iterable=True)
        register_stanza_plugin(xep_0004.FormField, xep_0122.FormValidation)

    def test_basic_validation(self):
        """Testing basic validation setting and getting."""
        msg = self.Message()
        form = msg['form']
        field = form.add_field(var='f1',
                               ftype='text-single',
                               label='Text',
                               desc='A text field',
                               required=True,
                               value='Some text!')

        validation = field['validate']
        validation['datatype'] = 'xs:string'
        validation.set_basic(True)

        self.check(msg, """
          <message>
            <x xmlns="jabber:x:data" type="form">
              <field var="f1" type="text-single" label="Text">
                <desc>A text field</desc>
                <required />
                <value>Some text!</value>
                <validate xmlns="http://jabber.org/protocol/xdata-validate" datatype="xs:string">
                    <basic/>
                </validate>
              </field>
            </x>
          </message>
        """)

        self.assertTrue(validation.get_basic())
        self.assertFalse(validation.get_open())
        self.assertFalse(validation.get_range())
        self.assertFalse(validation.get_regex())

    def test_open_validation(self):
        """Testing open validation setting and getting."""
        msg = self.Message()
        form = msg['form']
        field = form.add_field(var='f1',
                               ftype='text-single',
                               label='Text',
                               desc='A text field',
                               required=True,
                               value='Some text!')

        validation = field['validate']
        validation.set_open(True)

        self.check(msg, """
          <message>
            <x xmlns="jabber:x:data" type="form">
              <field var="f1" type="text-single" label="Text">
                <desc>A text field</desc>
                <required />
                <value>Some text!</value>
                <validate xmlns="http://jabber.org/protocol/xdata-validate">
                    <open />
                </validate>
              </field>
            </x>
          </message>
        """)

        self.assertFalse(validation.get_basic())
        self.assertTrue(validation.get_open())
        self.assertFalse(validation.get_range())
        self.assertFalse(validation.get_regex())

    def test_regex_validation(self):
        """Testing regex validation setting and getting."""
        msg = self.Message()
        form = msg['form']
        field = form.add_field(var='f1',
                               ftype='text-single',
                               label='Text',
                               desc='A text field',
                               required=True,
                               value='Some text!')

        regex_value = '[0-9]+'

        validation = field['validate']
        validation.set_regex(regex_value)

        self.check(msg, """
          <message>
            <x xmlns="jabber:x:data" type="form">
              <field var="f1" type="text-single" label="Text">
                <desc>A text field</desc>
                <required />
                <value>Some text!</value>
                <validate xmlns="http://jabber.org/protocol/xdata-validate">
                    <regex>[0-9]+</regex>
                </validate>
              </field>
            </x>
          </message>
        """)

        self.assertFalse(validation.get_basic())
        self.assertFalse(validation.get_open())
        self.assertFalse(validation.get_range())
        self.assertTrue(validation.get_regex())

        self.assertEqual(regex_value, validation.get_regex())

    def test_range_validation(self):
        """Testing range validation setting and getting."""
        msg = self.Message()
        form = msg['form']
        field = form.add_field(var='f1',
                               ftype='text-single',
                               label='Text',
                               desc='A text field',
                               required=True,
                               value='Some text!')

        validation = field['validate']
        validation.set_range(True, minimum=0, maximum=10)

        self.check(msg, """
          <message>
            <x xmlns="jabber:x:data" type="form">
              <field var="f1" type="text-single" label="Text">
                <desc>A text field</desc>
                <required />
                <value>Some text!</value>
                <validate xmlns="http://jabber.org/protocol/xdata-validate">
                    <range min="0" max="10" />
                </validate>
              </field>
            </x>
          </message>
        """)

        self.assertDictEqual(dict(minimum=str(0), maximum=str(10)), validation.get_range())

    def test_reported_field_validation(self):
        """
        Testing adding validation to the field when it's stored in the reported.
        :return:
        """
        msg = self.Message()
        form = msg['form']
        field = form.add_reported(var='f1', ftype='text-single', label='Text')
        validation = field['validate']
        validation.set_basic(True)

        form.add_item({'f1': 'Some text!'})

        self.check(msg, """
          <message>
            <x xmlns="jabber:x:data" type="form">
              <reported>
                <field var="f1" type="text-single" label="Text">
                  <validate xmlns="http://jabber.org/protocol/xdata-validate">
                    <basic />
                  </validate>
                </field>
              </reported>
              <item>
                <field var="f1">
                  <value>Some text!</value>
                </field>
              </item>
            </x>
          </message>
        """)


suite = unittest.TestLoader().loadTestsFromTestCase(TestDataForms)



import unittest
from random import randint
from slixmpp import JID
from slixmpp.test.integration import SlixIntegration


class TestMAM(SlixIntegration):
    async def asyncSetUp(self):
        await super().asyncSetUp()
        self.add_client(
            self.envjid('CI_ACCOUNT1'),
            self.envstr('CI_ACCOUNT1_PASSWORD'),
        )
        self.add_client(
            self.envjid('CI_ACCOUNT2'),
            self.envstr('CI_ACCOUNT2_PASSWORD'),
        )
        self.register_plugins(['xep_0313'])
        await self.connect_clients()

    async def test_mam_retrieve(self):
        """Make sure we can get messages from our archive"""
        # send messages first
        tok = randint(1, 999999)
        self.clients[0].make_message(
            mto=self.clients[1].boundjid,
            mbody=f'coucou {tok}'
        ).send()
        await self.clients[1].wait_until('message')
        self.clients[1].make_message(
            mto=self.clients[0].boundjid,
            mbody=f'coucou coucou {tok}',
        ).send()
        await self.clients[0].wait_until('message')

        # Get archive
        retrieve = self.clients[0]['xep_0313'].retrieve(
            with_jid=JID(self.envjid('CI_ACCOUNT2')),
            iterator=True,
            reverse=True,
            rsm={'max': 2}
        )
        msgs = []
        count = 0
        async for rsm in retrieve:
            for msg in rsm['mam']['results']:
                msgs.append(
                    msg['mam_result']['forwarded']['stanza']
                )
                count += 1
            if count >= 2:
                break

        self.assertEqual(msgs[0]['body'], f'coucou {tok}')
        self.assertEqual(msgs[1]['body'], f'coucou coucou {tok}')

    async def test_mam_iterate(self):
        """Make sure we can iterate over messages from our archive"""
        # send messages first
        tok = randint(1, 999999)
        self.clients[0].make_message(
            mto=self.clients[1].boundjid,
            mbody=f'coucou {tok}'
        ).send()
        await self.clients[1].wait_until('message')
        self.clients[1].make_message(
            mto=self.clients[0].boundjid,
            mbody='coucou coucou %s' % tok,
        ).send()
        await self.clients[0].wait_until('message')

        # Get archive
        retrieve = self.clients[0]['xep_0313'].iterate(
            with_jid=JID(self.envjid('CI_ACCOUNT2')),
            reverse=True,
            rsm={'max': 1}
        )
        msgs = []
        count = 0
        async for msg in retrieve:
            msgs.append(
                msg['mam_result']['forwarded']['stanza']
            )
            count += 1
            if count >= 2:
                break

        self.assertEqual(msgs[0]['body'], f'coucou coucou {tok}')
        self.assertEqual(msgs[1]['body'], f'coucou {tok}')


suite = unittest.TestLoader().loadTestsFromTestCase(TestMAM)




# Slixmpp: The Slick XMPP Library
# Copyright (C) 2012 Nathanael C. Fritz, Lance J.T. Stout
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.
from slixmpp.plugins.base import register_plugin

from slixmpp.plugins.xep_0334.stanza import Store, NoStore, NoPermanentStore, NoCopy
from slixmpp.plugins.xep_0334.hints import XEP_0334

register_plugin(XEP_0334)




# Slixmpp: The Slick XMPP Library
# Copyright (C) 2020 Mathieu Pasquet
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.
from typing import Iterable
from slixmpp import JID, Presence
from slixmpp.xmlstream import (
    ElementBase,
    register_stanza_plugin,
)

NS = 'urn:xmpp:rai:0'

class RAI(ElementBase):
    name = 'rai'
    plugin_attrib = 'rai'
    namespace = NS
    interfaces = {'activities'}

    def get_activities(self) -> Iterable[JID]:
        return [JID(el.xml.text) for el in self if isinstance(el, Activity)]

    def del_activities(self):
        for el in self.xml.findall('{%s}activity' % NS):
            self.xml.remove(el)

    def set_activities(self, activities: Iterable[JID]):
        self.del_activities()
        for jid in activities:
            act = Activity()
            act.xml.text = str(jid)
            self.append(act)


class Activity(ElementBase):
    name = 'activity'
    plugin_attrib = 'activity'
    namespace = NS


def register_plugins():
    register_stanza_plugin(RAI, Activity, iterable=True)
    register_stanza_plugin(Presence, RAI)




# Slixmpp: The Slick XMPP Library
# Copyright (C) 2010 Nathanael C. Fritz, Lance J.T. Stout
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.
import logging

from slixmpp import Message, Presence
from slixmpp.xmlstream import register_stanza_plugin
from slixmpp.plugins import BasePlugin
from slixmpp.plugins.xep_0033 import stanza, Addresses


class XEP_0033(BasePlugin):

    """
    XEP-0033: Extended Stanza Addressing
    """

    name = 'xep_0033'
    description = 'XEP-0033: Extended Stanza Addressing'
    dependencies = {'xep_0030'}
    stanza = stanza

    def plugin_init(self):
        register_stanza_plugin(Message, Addresses)
        register_stanza_plugin(Presence, Addresses)

    def plugin_end(self):
        self.xmpp['xep_0030'].del_feature(feature=Addresses.namespace)

    def session_bind(self, jid):
        self.xmpp['xep_0030'].add_feature(Addresses.namespace)




#!/usr/bin/env python3

# Slixmpp: The Slick XMPP Library
# Copyright (C) 2018 Emmanuel Gil Peyrot
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.

from typing import Optional

import sys
import logging
from pathlib import Path
from getpass import getpass
from argparse import ArgumentParser

import asyncio
import slixmpp
from slixmpp import JID
from slixmpp.exceptions import IqTimeout

log = logging.getLogger(__name__)


class HttpUpload(slixmpp.ClientXMPP):

    """
    A basic client asking an entity if they confirm the access to an HTTP URL.
    """

    def __init__(
        self,
        jid: JID,
        password: str,
        recipient: JID,
        filename: Path,
        domain: Optional[JID] = None,
        encrypted: bool = False,
    ):
        slixmpp.ClientXMPP.__init__(self, jid, password)

        self.recipient = recipient
        self.filename = filename
        self.domain = domain
        self.encrypted = encrypted

        self.add_event_handler("session_start", self.start)

    async def start(self, event):
        log.info('Uploading file %s...', self.filename)
        try:
            upload_file = self['xep_0363'].upload_file
            if self.encrypted and not self['xep_0454']:
                print(
                    'The xep_0454 module isn\'t available. '
                    'Ensure you have \'cryptography\' '
                    'from extras_require installed.',
                    file=sys.stderr,
                )
                return
            elif self.encrypted:
                upload_file = self['xep_0454'].upload_file
            url = await upload_file(
                self.filename, domain=self.domain, timeout=10,
            )
        except IqTimeout:
            raise TimeoutError('Could not send message in time')
        log.info('Upload success!')

        log.info('Sending file to %s', self.recipient)
        html = (
            f'<body xmlns="http://www.w3.org/1999/xhtml">'
            f'<a href="{url}">{url}</a></body>'
        )
        message = self.make_message(mto=self.recipient, mbody=url, mhtml=html)
        message['oob']['url'] = url
        message.send()
        self.disconnect()


if __name__ == '__main__':
    # Setup the command line arguments.
    parser = ArgumentParser()
    parser.add_argument("-q","--quiet", help="set logging to ERROR",
                        action="store_const",
                        dest="loglevel",
                        const=logging.ERROR,
                        default=logging.INFO)
    parser.add_argument("-d","--debug", help="set logging to DEBUG",
                        action="store_const",
                        dest="loglevel",
                        const=logging.DEBUG,
                        default=logging.INFO)

    # JID and password options.
    parser.add_argument("-j", "--jid", dest="jid",
                        help="JID to use")
    parser.add_argument("-p", "--password", dest="password",
                        help="password to use")

    # Other options.
    parser.add_argument("-r", "--recipient", required=True,
                        help="Recipient JID")
    parser.add_argument("-f", "--file", required=True,
                        help="File to send")
    parser.add_argument("--domain",
                        help="Domain to use for HTTP File Upload (leave out for your own server’s)")

    parser.add_argument("-e", "--encrypt", dest="encrypted",
                        help="Whether to encrypt", action="store_true",
                        default=False)

    args = parser.parse_args()

    # Setup logging.
    logging.basicConfig(level=args.loglevel,
                        format='%(levelname)-8s %(message)s')

    if args.jid is None:
        args.jid = JID(input("Username: "))
    if args.password is None:
        args.password = getpass("Password: ")

    domain = args.domain
    if domain is not None:
        domain = JID(domain)

    if args.encrypted:
        print(
            'You are using the --encrypt flag. '
            'Be aware that the transport being used is NOT end-to-end '
            'encrypted. The server will be able to decrypt the file.',
            file=sys.stderr,
        )

    xmpp = HttpUpload(
        jid=args.jid,
        password=args.password,
        recipient=JID(args.recipient),
        filename=Path(args.file),
        domain=domain,
        encrypted=args.encrypted,
    )
    xmpp.register_plugin('xep_0066')
    xmpp.register_plugin('xep_0071')
    xmpp.register_plugin('xep_0128')
    xmpp.register_plugin('xep_0363')
    try:
        xmpp.register_plugin('xep_0454')
    except slixmpp.plugins.base.PluginNotFound:
        log.error(
            'Could not load xep_0454. '
            'Ensure you have \'cryptography\' from extras_require installed.'
        )

    # Connect to the XMPP server and start processing XMPP stanzas.
    xmpp.connect()
    asyncio.get_event_loop().run_until_complete(xmpp.disconnected)



# Slixmpp: The Slick XMPP Library
# Copyright (C) 2025 Mathieu Pasquet
# This file is part of Slixmpp.
# See the file LICENSE for copying permissio
from slixmpp.plugins.base import register_plugin
from slixmpp.plugins.xep_0410.self_ping import XEP_0410, PingStatus

register_plugin(XEP_0410)




# Slixmpp: The Slick XMPP Library
# Copyright (C) 2020 Mathieu Pasquet
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.
from slixmpp.plugins.base import register_plugin

from slixmpp.plugins.xep_0377.stanza import Report, Text
from slixmpp.plugins.xep_0377.spam_reporting import XEP_0377


register_plugin(XEP_0377)




# slixmpp.clientxmpp
# ~~~~~~~~~~~~~~~~~~~~
# This module provides XMPP functionality that
# is specific to external server component connections.
# Part of Slixmpp: The Slick XMPP Library
# :copyright: (c) 2011 Nathanael C. Fritz
# :license: MIT, see LICENSE for more details
import logging
import hashlib

from asyncio import Future
from typing import Optional

from slixmpp import Message, Iq, Presence
from slixmpp.basexmpp import BaseXMPP
from slixmpp.stanza import Handshake
from slixmpp.stanza.error import Error
from slixmpp.xmlstream import XMLStream
from slixmpp.xmlstream.matcher import MatchXPath
from slixmpp.xmlstream.handler import Callback
from slixmpp.xmlstream.stanzabase import register_stanza_plugin

log = logging.getLogger(__name__)


class ComponentXMPP(BaseXMPP):

    """
    Slixmpp's basic XMPP server component.

    Use only for good, not for evil.

    :param jid: The JID of the component.
    :param secret: The secret or password for the component.
    :param host: The server accepting the component.
    :param port: The port used to connect to the server.
    :param plugin_config: A dictionary of plugin configurations.
    :param plugin_whitelist: A list of approved plugins that
                    will be loaded when calling
                    :meth:`~slixmpp.basexmpp.BaseXMPP.register_plugins()`.
    :param use_jc_ns: Indicates if the ``'jabber:client'`` namespace
                      should be used instead of the standard
                      ``'jabber:component:accept'`` namespace.
                      Defaults to ``False``.
    :param fix_error_ns: Fix the namespace of error stanzas.
        If you use ``use_jc_ns`` namespace, you probably want that, but
        it can be a problem if you use both a ClientXMPP and a ComponentXMPP
        in the same interpreter. This is ``False`` by default for backwards
        compatibility.
    """

    def __init__(self, jid, secret,
                 host=None, port=None, plugin_config=None,
                 plugin_whitelist=None, use_jc_ns=False,
                 fix_error_ns=False):

        if not plugin_whitelist:
            plugin_whitelist = []
        if not plugin_config:
            plugin_config = {}

        if use_jc_ns:
            default_ns = 'jabber:client'
        else:
            default_ns = 'jabber:component:accept'
        BaseXMPP.__init__(self, jid, default_ns)
        if fix_error_ns:
            self._fix_error_ns()

        self.auto_authorize = None
        self.stream_header = '<stream:stream %s %s to="%s">' % (
                'xmlns="jabber:component:accept"',
                'xmlns:stream="%s"' % self.stream_ns,
                jid)
        self.stream_footer = "</stream:stream>"
        self.server_host = host
        self.server_port = port
        self.secret = secret

        self.plugin_config = plugin_config
        self.plugin_whitelist = plugin_whitelist
        self.is_component = True

        self.sessionstarted = False

        self.register_handler(
                Callback('Handshake',
                         MatchXPath('{jabber:component:accept}handshake'),
                         self._handle_handshake))
        self.add_event_handler('presence_probe',
                               self._handle_probe)

    def _fix_error_ns(self):
        Error.namespace = self.default_ns
        for st in Message, Iq, Presence:
            register_stanza_plugin(st, Error)

    def connect(self, host: Optional[str] = None, port: int = 0, use_ssl: Optional[bool] = None,
                force_starttls: Optional[bool] = None,
                disable_starttls: Optional[bool] = None) -> Future:
        """Connect to the server.


        :param host: The name of the desired server for the connection.
                     Defaults to :attr:`server_host`.
        :param port: Port to connect to on the server.
                     Defauts to :attr:`server_port`.
        :param use_ssl: Flag indicating if SSL should be used by connecting
                        directly to a port using SSL.
        :param force_starttls: UNUSED
        :param disable_starttls: UNUSED
        """
        if host is not None:
            self.server_host = host
        if port:
            self.server_port = port

        self.server_name = self.boundjid.host

        self.init_plugins()

        log.debug("Connecting to %s:%s", host, port)
        return XMLStream.connect(self, host=self.server_host, port=self.server_port, use_ssl=use_ssl)

    def incoming_filter(self, xml):
        """
        Pre-process incoming XML stanzas by converting any
        ``'jabber:client'`` namespaced elements to the component's
        default namespace.

        :param xml: The XML stanza to pre-process.
        """
        if xml.tag.startswith('{jabber:client}'):
            xml.tag = xml.tag.replace('jabber:client', self.default_ns)
        return xml

    def start_stream_handler(self, xml):
        """
        Once the streams are established, attempt to handshake
        with the server to be accepted as a component.

        :param xml: The incoming stream's root element.
        """
        BaseXMPP.start_stream_handler(self, xml)

        # Construct a hash of the stream ID and the component secret.
        sid = xml.get('id', '')
        pre_hash = bytes('%s%s' % (sid, self.secret), 'utf-8')

        handshake = Handshake()
        handshake['value'] = hashlib.sha1(pre_hash).hexdigest().lower()

        self.send(handshake)

    def _handle_handshake(self, xml):
        """The handshake has been accepted.

        :param xml: The reply handshake stanza.
        """
        self.session_bind_event.set()
        self.sessionstarted = True
        self.event('session_bind', self.boundjid)
        self.event('session_start')

    def _handle_probe(self, pres):
        self.roster[pres['to']][pres['from']].handle_probe(pres)




# slixmpp: The Slick XMPP Library
# Copyright (C) 2011  Nathanael C. Fritz
# This file is part of slixmpp.
# See the file LICENSE for copying permission.
import logging
import zlib


from slixmpp.stanza import StreamFeatures
from slixmpp.xmlstream import RestartStream, register_stanza_plugin, ElementBase, StanzaBase
from slixmpp.xmlstream.matcher import *
from slixmpp.xmlstream.handler import *
from slixmpp.plugins import BasePlugin, register_plugin

log = logging.getLogger(__name__)


class Compression(ElementBase):
    name = 'compression'
    namespace = 'http://jabber.org/features/compress'
    interfaces = {'methods'}
    plugin_attrib = 'compression'
    plugin_tag_map = {}
    plugin_attrib_map = {}

    def get_methods(self):
        methods = []
        for method in self.xml.findall('{%s}method' % self.namespace):
            methods.append(method.text)
        return methods


class Compress(StanzaBase):
    name = 'compress'
    namespace = 'http://jabber.org/protocol/compress'
    interfaces = {'method'}
    sub_interfaces = interfaces
    plugin_attrib = 'compress'
    plugin_tag_map = {}
    plugin_attrib_map = {}

    def setup(self, xml):
        StanzaBase.setup(self, xml)
        self.xml.tag = self.tag_name()


class Compressed(StanzaBase):
    name = 'compressed'
    namespace = 'http://jabber.org/protocol/compress'
    interfaces = set()
    plugin_tag_map = {}
    plugin_attrib_map = {}

    def setup(self, xml):
        StanzaBase.setup(self, xml)
        self.xml.tag = self.tag_name()




class ZlibSocket(object):

    def __init__(self, socketobj):
        self.__socket = socketobj
        self.compressor = zlib.compressobj()
        self.decompressor = zlib.decompressobj(zlib.MAX_WBITS)

    def __getattr__(self, name):
        return getattr(self.__socket, name)

    def send(self, data):
        sentlen = len(data)
        data = self.compressor.compress(data)
        data += self.compressor.flush(zlib.Z_SYNC_FLUSH)
        log.debug(b'>>> (compressed)' + (data.encode("hex")))
        #return self.__socket.send(data)
        sentactuallen = self.__socket.send(data)
        assert(sentactuallen == len(data))

        return sentlen

    def recv(self, *args, **kwargs):
        data = self.__socket.recv(*args, **kwargs)
        log.debug(b'<<< (compressed)' + data.encode("hex"))
        return self.decompressor.decompress(self.decompressor.unconsumed_tail + data)


class XEP_0138(BasePlugin):
    """
    XEP-0138: Compression
    """
    name = "xep_0138"
    description = "XEP-0138: Compression"
    dependencies = {"xep_0030"}

    def plugin_init(self):
        self.xep = '0138'
        self.description = 'Stream Compression (Generic)'

        self.compression_methods = {'zlib': True}

        register_stanza_plugin(StreamFeatures, Compression)
        self.xmpp.register_stanza(Compress)
        self.xmpp.register_stanza(Compressed)

        self.xmpp.register_handler(
                Callback('Compressed',
                    StanzaPath('compressed'),
                    self._handle_compressed,
                    instream=True))

        self.xmpp.register_feature('compression',
                self._handle_compression,
                restart=True,
                order=self.config.get('order', 5))

    def register_compression_method(self, name, handler):
        self.compression_methods[name] = handler

    def _handle_compression(self, features):
        for method in features['compression']['methods']:
            if method in self.compression_methods:
                log.info('Attempting to use %s compression' % method)
                c = Compress(self.xmpp)
                c['method'] = method
                c.send(now=True)
                return True
        return False

    def _handle_compressed(self, stanza):
        self.xmpp.features.add('compression')
        log.debug('Stream Compressed!')
        compressed_socket = ZlibSocket(self.xmpp.socket)
        self.xmpp.set_socket(compressed_socket)
        raise RestartStream()

    def _handle_failure(self, stanza):
        pass

xep_0138 = XEP_0138
register_plugin(XEP_0138)



# Slixmpp: The Slick XMPP Library
# This file is part of Slixmpp
# See the file LICENSE for copying permission
import asyncio
import socket
import logging

from typing import (
    Optional,
    IO,
    Union,
)

from slixmpp import JID
from slixmpp.stanza import Iq, Message
from slixmpp.exceptions import XMPPError, IqTimeout


log = logging.getLogger(__name__)


class IBBytestream(object):
    """XEP-0047 Stream abstraction. Created by the ibb plugin automatically.

    Provides send methods and triggers :term:`ibb_stream_data` events.
    """

    def __init__(self, xmpp, sid: str, block_size: int, jid: JID, peer: JID,
                 use_messages: bool = False):
        self.xmpp = xmpp
        self.sid = sid
        self.block_size = block_size
        self.use_messages = use_messages

        if jid is None:
            jid = xmpp.boundjid
        self.self_jid = jid
        self.peer_jid = peer

        self.send_seq = -1
        self.recv_seq = -1

        self.stream_started = False
        self.stream_in_closed = False
        self.stream_out_closed = False

        self.recv_queue = asyncio.Queue()

    async def send(self, data: bytes, timeout: Optional[int] = None) -> int:
        """Send a single block of data.

        :param data: Data to send (will be truncated if above block size).
        :returns: Number of bytes sent.
        """
        if not self.stream_started or self.stream_out_closed:
            raise socket.error
        if len(data) > self.block_size:
            data = data[:self.block_size]
        self.send_seq = (self.send_seq + 1) % 65536
        seq = self.send_seq
        if self.use_messages:
            msg = self.xmpp.Message()
            msg['to'] = self.peer_jid
            msg['from'] = self.self_jid
            msg['id'] = self.xmpp.new_id()
            msg['ibb_data']['sid'] = self.sid
            msg['ibb_data']['seq'] = seq
            msg['ibb_data']['data'] = data
            msg.send()
        else:
            iq = self.xmpp.Iq()
            iq['type'] = 'set'
            iq['to'] = self.peer_jid
            iq['from'] = self.self_jid
            iq['ibb_data']['sid'] = self.sid
            iq['ibb_data']['seq'] = seq
            iq['ibb_data']['data'] = data
            await iq.send(timeout=timeout)
        return len(data)

    async def sendall(self, data: bytes, timeout: Optional[int] = None):
        """Send all the contents of ``data`` in chunks.

        :param data: Raw data to send.
        """
        sent_len = 0
        while sent_len < len(data):
            sent_len += await self.send(data[sent_len:sent_len+self.block_size], timeout=timeout)

    async def gather(self, max_data: Optional[int] = None, timeout: int = 3600) -> bytes:
        """Gather all data sent on a stream until it is closed, and return it.

        .. versionadded:: 1.8.0

        :param max_data: Max number of bytes to receive. (received data may be
                         over this limit depending on block_size)
        :param timeout: Timeout after which an error will be raised.
        :raises .IqTimeout: If the timeout is reached.
        :returns: All bytes accumulated in the stream.
        """
        result = b''
        while not self.recv_queue.empty():
            result += self.recv_queue.get_nowait()
            if max_data and len(result) > max_data:
                return result
        if self.stream_in_closed:
            return result

        end_future = asyncio.Future()

        def on_close(stream):
            if stream is self:
                end_future.set_result(True)

        def on_data(stream):
            nonlocal result
            if stream is self:
                result += stream.read()
                if max_data and len(result) > max_data:
                    end_future.set_result(True)

        self.xmpp.add_event_handler('ibb_stream_end', on_close)
        self.xmpp.add_event_handler('ibb_stream_data', on_data)
        try:
            await asyncio.wait_for(end_future, timeout)
        except asyncio.TimeoutError:
            raise IqTimeout(result)
        finally:
            self.xmpp.del_event_handler('ibb_stream_end', on_close)
            self.xmpp.del_event_handler('ibb_stream_data', on_data)
        return result

    async def sendfile(self, file: IO[bytes], timeout: Optional[int] = None):
        """Send the contents of a file over the wire, in chunks.

        :param file: The opened file (or file-like) object, in bytes mode."""
        while True:
            data = file.read(self.block_size)
            if not data:
                break
            await self.send(data, timeout=timeout)

    def _recv_data(self, stanza: Union[Message, Iq]):
        new_seq = stanza['ibb_data']['seq']
        if new_seq != (self.recv_seq + 1) % 65536:
            self.close()
            raise XMPPError('unexpected-request')
        self.recv_seq = new_seq

        data = stanza['ibb_data']['data']
        if len(data) > self.block_size:
            self.close()
            raise XMPPError('not-acceptable')

        self.recv_queue.put_nowait(data)
        self.xmpp.event('ibb_stream_data', self)

        if isinstance(stanza, Iq):
            stanza.reply().send()

    def recv(self, *args, **kwargs):
        return self.read()

    def read(self):
        if not self.stream_started or self.stream_in_closed:
            raise socket.error
        return self.recv_queue.get_nowait()

    def close(self, timeout: Optional[int] = None) -> asyncio.Future:
        """Close the stream."""
        iq = self.xmpp.Iq()
        iq['type'] = 'set'
        iq['to'] = self.peer_jid
        iq['from'] = self.self_jid
        iq['ibb_close']['sid'] = self.sid
        self.stream_out_closed = True
        def _close_stream(_):
            self.stream_in_closed = True
        future = iq.send(timeout=timeout, callback=_close_stream)
        self.xmpp.event('ibb_stream_end', self)
        return future

    def _closed(self, iq: Iq):
        self.stream_in_closed = True
        self.stream_out_closed = True
        iq.reply().send()
        self.xmpp.event('ibb_stream_end', self)

    def makefile(self, *args, **kwargs):
        return self

    def connect(*args, **kwargs):
        return None

    def shutdown(self, *args, **kwargs):
        return None



import unittest
from slixmpp.test.integration import SlixIntegration


class TestReconnect(SlixIntegration):
    async def asyncSetUp(self):
        await super().asyncSetUp()
        self.add_client(
            self.envjid('CI_ACCOUNT1'),
            self.envstr('CI_ACCOUNT1_PASSWORD'),
        )
        await self.connect_clients()

    async def test_disconnect_connect(self):
        """Check we can disconnect and connect again"""
        await self.clients[0].disconnect()
        self.clients[0].connect()
        await self.clients[0].wait_until('session_start')

    async def test_reconnect(self):
        """Check we can reconnect()"""
        self.clients[0].reconnect()
        await self.clients[0].wait_until("session_start")

suite = unittest.TestLoader().loadTestsFromTestCase(TestReconnect)



import unittest
from slixmpp import JID
from slixmpp.test.integration import SlixIntegration


class TestBlocking(SlixIntegration):
    async def asyncSetUp(self):
        await super().asyncSetUp()
        self.add_client(
            self.envjid('CI_ACCOUNT1'),
            self.envstr('CI_ACCOUNT1_PASSWORD'),
        )
        self.register_plugins(['xep_0191'])
        await self.connect_clients()

    async def test_blocking(self):
        """Check we can block, unblock, and list blocked"""
        await self.clients[0]['xep_0191'].block(
            [JID('toto@example.com'), JID('titi@example.com')]
        )
        blocked = {JID('toto@example.com'), JID('titi@example.com')}
        iq = await self.clients[0]['xep_0191'].get_blocked()
        self.assertEqual(iq['blocklist']['items'], blocked)

        info = await self.clients[0]['xep_0191'].unblock(
            blocked,
        )
        iq = await self.clients[0]['xep_0191'].get_blocked()
        self.assertEqual(len(iq['blocklist']['items']), 0)


suite = unittest.TestLoader().loadTestsFromTestCase(TestBlocking)




# Slixmpp: The Slick XMPP Library
# Copyright (C) 2011 Nathanael C. Fritz, Lance J.T. Stout
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.
import datetime as dt

from slixmpp.plugins import BasePlugin, register_plugin


# =====================================================================
# To make it easier for stanzas without direct access to plugin objects
# to use the XEP-0082 utility methods, we will define them as top-level
# functions and then just reference them in the plugin itself.

def parse(time_str):
    """
    Convert a string timestamp into a datetime object.

    Arguments:
        time_str -- A formatted timestamp string.
    """
    try:
        return dt.datetime.strptime(time_str, '%Y-%m-%dT%H:%M:%S.%f%z')
    except ValueError:
        return dt.datetime.strptime(time_str, '%Y-%m-%dT%H:%M:%S%z')


def format_date(time_obj):
    """
    Return a formatted string version of a date object.

    Format:
        YYYY-MM-DD

    Arguments:
        time_obj -- A date or datetime object.
    """
    if isinstance(time_obj, dt.datetime):
        time_obj = time_obj.date()
    return time_obj.isoformat()


def format_time(time_obj):
    """
    Return a formatted string version of a time object.

    format:
        hh:mm:ss[.sss][TZD]

    arguments:
        time_obj -- A time or datetime object.
    """
    if isinstance(time_obj, dt.datetime):
        time_obj = time_obj.timetz()
    timestamp = time_obj.isoformat()
    if time_obj.tzinfo == dt.timezone.utc:
        timestamp = timestamp[:-6]
        return '%sZ' % timestamp
    return timestamp


def format_datetime(time_obj):
    """
    Return a formatted string version of a datetime object.

    Format:
        YYYY-MM-DDThh:mm:ss[.sss]TZD

    arguments:
        time_obj -- A datetime object.
    """
    timestamp = time_obj.isoformat('T')
    if time_obj.tzinfo == dt.timezone.utc:
        timestamp = timestamp[:-6]
        return '%sZ' % timestamp
    return timestamp


def date(year=None, month=None, day=None, obj=False):
    """
    Create a date only timestamp for the given instant.

    Unspecified components default to their current counterparts.

    Arguments:
        year   -- Integer value of the year (4 digits)
        month  -- Integer value of the month
        day    -- Integer value of the day of the month.
        obj    -- If True, return the date object instead
                  of a formatted string. Defaults to False.
    """
    today = dt.datetime.utcnow()
    if year is None:
        year = today.year
    if month is None:
        month = today.month
    if day is None:
        day = today.day
    value = dt.date(year, month, day)
    if obj:
        return value
    return format_date(value)


def time(hour=None, min=None, sec=None, micro=None, offset=None, obj=False):
    """
    Create a time only timestamp for the given instant.

    Unspecified components default to their current counterparts.

    Arguments:
        hour   -- Integer value of the hour.
        min    -- Integer value of the number of minutes.
        sec    -- Integer value of the number of seconds.
        micro  -- Integer value of the number of microseconds.
        offset -- Either a positive or negative number of seconds
                  to offset from UTC to match a desired timezone,
                  or a tzinfo object.
        obj    -- If True, return the time object instead
                  of a formatted string. Defaults to False.
    """
    now = dt.datetime.utcnow()
    if hour is None:
        hour = now.hour
    if min is None:
        min = now.minute
    if sec is None:
        sec = now.second
    if micro is None:
        micro = now.microsecond
    if offset in (None, 0):
        offset = dt.timezone.utc
    elif not isinstance(offset, dt.tzinfo):
        offset = dt.timezone(dt.timedelta(seconds=offset))
    value = dt.time(hour, min, sec, micro, offset)
    if obj:
        return value
    return format_time(value)


def datetime(year=None, month=None, day=None, hour=None,
             min=None, sec=None, micro=None, offset=None,
             separators=True, obj=False):
    """
    Create a datetime timestamp for the given instant.

    Unspecified components default to their current counterparts.

    Arguments:
        year   -- Integer value of the year (4 digits)
        month  -- Integer value of the month
        day    -- Integer value of the day of the month.
        hour   -- Integer value of the hour.
        min    -- Integer value of the number of minutes.
        sec    -- Integer value of the number of seconds.
        micro  -- Integer value of the number of microseconds.
        offset -- Either a positive or negative number of seconds
                  to offset from UTC to match a desired timezone,
                  or a tzinfo object.
        obj    -- If True, return the datetime object instead
                  of a formatted string. Defaults to False.
    """
    now = dt.datetime.utcnow()
    if year is None:
        year = now.year
    if month is None:
        month = now.month
    if day is None:
        day = now.day
    if hour is None:
        hour = now.hour
    if min is None:
        min = now.minute
    if sec is None:
        sec = now.second
    if micro is None:
        micro = now.microsecond
    if offset in (None, 0):
        offset = dt.timezone.utc
    elif not isinstance(offset, dt.tzinfo):
        offset = dt.timezone(dt.timedelta(seconds=offset))

    value = dt.datetime(year, month, day, hour,
                       min, sec, micro, offset)
    if obj:
        return value
    return format_datetime(value)


class XEP_0082(BasePlugin):

    """
    XEP-0082: XMPP Date and Time Profiles

    XMPP uses a subset of the formats allowed by ISO 8601 as a matter of
    pragmatism based on the relatively few formats historically used by
    the XMPP.

    Also see <http://www.xmpp.org/extensions/xep-0082.html>.

    Methods:
        date            -- Create a time stamp using the Date profile.
        datetime        -- Create a time stamp using the DateTime profile.
        time            -- Create a time stamp using the Time profile.
        format_date     -- Format an existing date object.
        format_datetime -- Format an existing datetime object.
        format_time     -- Format an existing time object.
        parse           -- Convert a time string into a Python datetime object.
    """

    name = 'xep_0082'
    description = 'XEP-0082: XMPP Date and Time Profiles'
    dependencies = set()

    def plugin_init(self):
        """Start the XEP-0082 plugin."""
        self.date = date
        self.datetime = datetime
        self.time = time
        self.format_date = format_date
        self.format_datetime = format_datetime
        self.format_time = format_time
        self.parse = parse


register_plugin(XEP_0082)




# Slixmpp: The Slick XMPP Library
# Copyright (C) 2020 Mathieu Pasquet
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.
from slixmpp.xmlstream import ET, ElementBase


class Report(ElementBase):
    """
    A spam/abuse report.

    Example sub stanza:
    ::

        <report xmlns="urn:xmpp:reporting:1" reason="urn:xmpp:reporting:abuse">
          <text xml:lang="en">
            Never came trouble to my house like this.
          </text>
        </report>

    The reason attribute is mandatory.

    """
    name = "report"
    namespace = "urn:xmpp:reporting:1"
    plugin_attrib = "report"
    interfaces = ("text", "reason")
    sub_interfaces = {'text'}


class Text(ElementBase):
    name = "text"
    plugin_attrib = "text"
    namespace = "urn:xmpp:reporting:1"



#!/usr/bin/env python3

# Slixmpp: The Slick XMPP Library
# Copyright (C) 2015 Emmanuel Gil Peyrot
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.

import logging
from getpass import getpass
from argparse import ArgumentParser

import asyncio
import slixmpp
from slixmpp.exceptions import XMPPError

log = logging.getLogger(__name__)


class AnswerConfirm(slixmpp.ClientXMPP):

    """
    A basic client demonstrating how to confirm or deny an HTTP request.
    """

    def __init__(self, jid, password, trusted):
        slixmpp.ClientXMPP.__init__(self, jid, password)

        self.add_event_handler("http_confirm", self.confirm)
        self.add_event_handler("session_start", self.start)

    def start(self, *args):
        self.make_presence().send()

    def prompt(self, stanza):
        confirm = stanza['confirm']
        print('Received confirm request %s from %s to access %s using '
                 'method %s' % (
                     confirm['id'], stanza['from'], confirm['url'],
                     confirm['method'])
                )
        result = input("Do you accept (y/N)? ")
        return 'y' == result.lower()

    def confirm(self, stanza):
        if self.prompt(stanza):
            reply = stanza.reply()
        else:
            reply = stanza.reply()
            reply.enable('error')
            reply['error']['type'] = 'auth'
            reply['error']['code'] = '401'
            reply['error']['condition'] = 'not-authorized'
        reply.append(stanza['confirm'])
        reply.send()

if __name__ == '__main__':
    # Setup the command line arguments.
    parser = ArgumentParser()
    parser.add_argument("-q","--quiet", help="set logging to ERROR",
                        action="store_const",
                        dest="loglevel",
                        const=logging.ERROR,
                        default=logging.INFO)
    parser.add_argument("-d","--debug", help="set logging to DEBUG",
                        action="store_const",
                        dest="loglevel",
                        const=logging.DEBUG,
                        default=logging.INFO)

    # JID and password options.
    parser.add_argument("-j", "--jid", dest="jid",
                        help="JID to use")
    parser.add_argument("-p", "--password", dest="password",
                        help="password to use")

    # Other options.
    parser.add_argument("-t", "--trusted", nargs='*',
                        help="List of trusted JIDs")

    args = parser.parse_args()

    # Setup logging.
    logging.basicConfig(level=args.loglevel,
                        format='%(levelname)-8s %(message)s')

    if args.jid is None:
        args.jid = input("Username: ")
    if args.password is None:
        args.password = getpass("Password: ")

    xmpp = AnswerConfirm(args.jid, args.password, args.trusted)
    xmpp.register_plugin('xep_0070')

    # Connect to the XMPP server and start processing XMPP stanzas.
    xmpp.connect()
    asyncio.get_event_loop().run_forever()



# Slixmpp: The Slick XMPP Library
# Copyright (C) 2011 Nathanael C. Fritz, Lance J.T. Stout
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.

import logging

from asyncio import Future
from typing import Optional
from slixmpp.plugins.base import BasePlugin
from slixmpp.plugins.xep_0118 import stanza, UserTune


log = logging.getLogger(__name__)


class XEP_0118(BasePlugin):

    """
    XEP-0118: User Tune
    """

    name = 'xep_0118'
    description = 'XEP-0118: User Tune'
    dependencies = {'xep_0163'}
    stanza = stanza

    def plugin_end(self):
        self.xmpp['xep_0030'].del_feature(feature=UserTune.namespace)
        self.xmpp['xep_0163'].remove_interest(UserTune.namespace)

    def session_bind(self, jid):
        self.xmpp['xep_0163'].register_pep('user_tune', UserTune)

    def publish_tune(self, *, artist: Optional[str] = None,
                     length: Optional[int] =None, rating: Optional[int] = None,
                     source: Optional[str] = None, title: Optional[str] = None,
                     track: Optional[str] = None, uri: Optional[str] = None,
                     **pubsubkwargs) -> Future:
        """
        Publish the user's current tune.

        :param artist: The artist or performer of the song.
        :param length: The length of the song in seconds.
        :param rating: The user's rating of the song (from 1 to 10)
        :param source: The album name, website, or other source of the song.
        :param title: The title of the song.
        :param track: The song's track number, or other unique identifier.
        :param uri: A URL to more information about the song.
        """
        tune = UserTune()
        tune['artist'] = artist
        tune['length'] = length
        tune['rating'] = rating
        tune['source'] = source
        tune['title'] = title
        tune['track'] = track
        tune['uri'] = uri
        return self.xmpp['xep_0163'].publish(
            tune,
            node=UserTune.namespace,
            **pubsubkwargs
        )

    def stop(self, **pubsubkwargs) -> Future:
        """
        Clear existing user tune information to stop notifications.
        """
        tune = UserTune()
        return self.xmpp['xep_0163'].publish(
            tune,
            node=UserTune.namespace,
            **pubsubkwargs
        )



import unittest
from slixmpp.test.integration import SlixIntegration


class TestDisco(SlixIntegration):
    async def asyncSetUp(self):
        await super().asyncSetUp()
        self.add_client(
            self.envjid('CI_ACCOUNT1'),
            self.envstr('CI_ACCOUNT1_PASSWORD'),
        )
        self.add_client(
            self.envjid('CI_ACCOUNT2'),
            self.envstr('CI_ACCOUNT2_PASSWORD'),
        )
        self.register_plugins(['xep_0030'])
        await self.connect_clients()

    async def test_features(self):
        """Check we can add, get and delete a feature"""
        self.clients[0]['xep_0030'].add_feature('urn:xmpp:fake:0')
        info = await self.clients[1]['xep_0030'].get_info(
            self.clients[0].boundjid.full
        )
        self.assertIn('urn:xmpp:fake:0', info['disco_info']['features'])
        self.clients[0]['xep_0030'].del_feature(feature='urn:xmpp:fake:0')
        info = await self.clients[1]['xep_0030'].get_info(
            self.clients[0].boundjid.full
        )
        self.assertNotIn('urn:xmpp:fake:0', info['disco_info']['features'])


suite = unittest.TestLoader().loadTestsFromTestCase(TestDisco)



import time
import unittest
from slixmpp.test import SlixTest


class TestEvents(SlixTest):

    def setUp(self):
        self.stream_start()

    def testEventHappening(self):
        """Test handler working"""
        happened = []

        def handletestevent(event):
            happened.append(True)

        self.xmpp.add_event_handler("test_event", handletestevent)
        self.xmpp.event("test_event")
        self.xmpp.event("test_event")

        msg = "Event was not triggered the correct number of times: %s"
        self.assertTrue(happened == [True, True], msg)

    def testDelEvent(self):
        """Test handler working, then deleted and not triggered"""
        happened = []

        def handletestevent(event):
            happened.append(True)

        self.xmpp.add_event_handler("test_event", handletestevent)
        self.xmpp.event("test_event", {})

        self.xmpp.del_event_handler("test_event", handletestevent)

        # Should not trigger because it was deleted
        self.xmpp.event("test_event", {})

        msg = "Event was not triggered the correct number of times: %s"
        self.assertTrue(happened == [True], msg % happened)

    def testAddDelAddEvent(self):
        """Test adding, then removing, then adding an event handler."""
        happened = []

        def handletestevent(event):
            happened.append(True)

        self.xmpp.add_event_handler("test_event", handletestevent)
        self.xmpp.event("test_event", {})

        self.xmpp.del_event_handler("test_event", handletestevent)
        # Should not trigger because it was deleted
        self.xmpp.event("test_event", {})

        self.xmpp.add_event_handler("test_event", handletestevent)
        self.xmpp.event("test_event", {})

        msg = "Event was not triggered the correct number of times: %s"
        self.assertTrue(happened == [True, True], msg % happened)

    def testDisposableEvent(self):
        """Test disposable handler working, then not being triggered again."""
        happened = []

        def handletestevent(event):
            happened.append(True)

        self.xmpp.add_event_handler("test_event", handletestevent,
                                    disposable=True)
        self.xmpp.event("test_event", {})

        # Should not trigger because it was deleted
        self.xmpp.event("test_event", {})

        msg = "Event was not triggered the correct number of times: %s"
        self.assertTrue(happened == [True], msg % happened)


suite = unittest.TestLoader().loadTestsFromTestCase(TestEvents)



import unittest
from slixmpp.test import SlixTest
from slixmpp.xmlstream.stanzabase import ET, StanzaBase


class TestStanzaBase(SlixTest):

    def testTo(self):
        """Test the 'to' interface of StanzaBase."""
        stanza = StanzaBase()
        stanza['to'] = 'user@example.com'
        self.assertTrue(str(stanza['to']) == 'user@example.com',
            "Setting and retrieving stanza 'to' attribute did not work.")

    def testFrom(self):
        """Test the 'from' interface of StanzaBase."""
        stanza = StanzaBase()
        stanza['from'] = 'user@example.com'
        self.assertTrue(str(stanza['from']) == 'user@example.com',
            "Setting and retrieving stanza 'from' attribute did not work.")

    def testPayload(self):
        """Test the 'payload' interface of StanzaBase."""
        stanza = StanzaBase()
        self.assertTrue(stanza['payload'] == [],
            "Empty stanza does not have an empty payload.")

        stanza['payload'] = ET.Element("{foo}foo")
        self.assertTrue(len(stanza['payload']) == 1,
            "Stanza contents and payload do not match.")

        stanza['payload'] = ET.Element('{bar}bar')
        self.assertTrue(len(stanza['payload']) == 2,
            "Stanza payload was not appended.")

        del stanza['payload']
        self.assertTrue(stanza['payload'] == [],
            "Stanza payload not cleared after deletion.")

        stanza['payload'] = [ET.Element('{foo}foo'),
                             ET.Element('{bar}bar')]
        self.assertTrue(len(stanza['payload']) == 2,
            "Adding multiple elements to stanza's payload did not work.")

    def testClear(self):
        """Test clearing a stanza."""
        stanza = StanzaBase()
        stanza['to'] = 'user@example.com'
        stanza['payload'] = ET.Element("{foo}foo")
        stanza.clear()

        self.assertTrue(stanza['payload'] == [],
            "Stanza payload was not cleared after calling .clear()")
        self.assertTrue(str(stanza['to']) == "user@example.com",
            "Stanza attributes were not preserved after calling .clear()")

    def testReply(self):
        """Test creating a reply stanza."""
        stanza = StanzaBase()
        stanza['to'] = "recipient@example.com"
        stanza['from'] = "sender@example.com"
        stanza['payload'] = ET.Element("{foo}foo")

        stanza = stanza.reply()

        self.assertTrue(str(stanza['to'] == "sender@example.com"),
            "Stanza reply did not change 'to' attribute.")
        self.assertTrue(stanza['payload'] == [],
            "Stanza reply did not empty stanza payload.")


suite = unittest.TestLoader().loadTestsFromTestCase(TestStanzaBase)



#!/usr/bin/env python3

# Slixmpp: The Slick XMPP Library
# Copyright (C) 2011  Dann Martens
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.

import asyncio
import logging
from argparse import ArgumentParser
from getpass import getpass
from slixmpp.jid import JID
from slixmpp.plugins.xep_0009.remote import Endpoint, remote, Remote, \
    ANY_ALL

class Thermostat(Endpoint):

    def FQN(self):
        return 'thermostat'

    def __init__(self, initial_temperature):
        self._temperature = initial_temperature
        self._event = asyncio.Event()

    @remote
    def set_temperature(self, temperature):
        print("Setting temperature to %s" % temperature)
        self._temperature = temperature

    @remote
    def get_temperature(self):
        return self._temperature

    @remote(False)
    def release(self):
        self._event.set()

    async def wait_for_release(self):
        await self._event.wait()


async def main(jid: JID, password: str):
    session = await Remote.new_session(jid, password)
    thermostat = session.new_handler(ANY_ALL, Thermostat, 18)
    await thermostat.wait_for_release()
    session.close()

if __name__ == '__main__':
    # Setup the command line arguments.
    parser = ArgumentParser()

    # Output verbosity options.
    parser.add_argument("-q", "--quiet", help="set logging to ERROR",
                        action="store_const", dest="loglevel",
                        const=logging.ERROR, default=logging.INFO)
    parser.add_argument("-d", "--debug", help="set logging to DEBUG",
                        action="store_const", dest="loglevel",
                        const=logging.DEBUG, default=logging.INFO)

    # JID and password options.
    parser.add_argument("-j", "--jid", dest="jid",
                        help="JID to use")
    parser.add_argument("-p", "--password", dest="password",
                        help="password to use")

    args = parser.parse_args()

    # Setup logging.
    logging.basicConfig(level=args.loglevel,
                        format='%(levelname)-8s %(message)s')

    if args.jid is None:
        args.jid = input("Username: ")
    if args.password is None:
        args.password = getpass("Password: ")

    asyncio.run(main(JID(args.jid), args.password))




# Slixmpp: The Slick XMPP Library
# Copyright (C) 2013 Nathanael C. Fritz, Lance J.T. Stout
# This file is part of slixmpp.
# See the file LICENSE for copying permission.
from slixmpp.xmlstream import register_stanza_plugin
from slixmpp.plugins import BasePlugin
from slixmpp.plugins.google.auth import stanza


class GoogleAuth(BasePlugin):

    """
    Google: Auth Extensions (JID Domain Discovery, OAuth2)

    Also see:
        <https://developers.google.com/talk/jep_extensions/jid_domain_change>
        <https://developers.google.com/talk/jep_extensions/oauth>
    """

    name = 'google_auth'
    description = 'Google: Auth Extensions (JID Domain Discovery, OAuth2)'
    dependencies = set(['feature_mechanisms'])
    stanza = stanza

    def plugin_init(self):
        self.xmpp.namespace_map['http://www.google.com/talk/protocol/auth'] = 'ga'

        register_stanza_plugin(self.xmpp['feature_mechanisms'].stanza.Auth,
                               stanza.GoogleAuth)

        self.xmpp.add_filter('out', self._auth)

    def plugin_end(self):
        self.xmpp.del_filter('out', self._auth)

    def _auth(self, stanza):
        if isinstance(stanza, self.xmpp['feature_mechanisms'].stanza.Auth):
            stanza.stream = self.xmpp
            stanza['google']['client_uses_full_bind_result'] = True
            if stanza['mechanism'] == 'X-OAUTH2':
                stanza['google']['service'] = 'oauth2'
            print(stanza)
        return stanza




# slixmpp.jid
# ~~~~~~~~~~~~~~~~~~~~~~~
# This module allows for working with Jabber IDs (JIDs).
# Part of Slixmpp: The Slick XMPP Library
# :copyright: (c) 2011 Nathanael C. Fritz
# :license: MIT, see LICENSE for more details
from __future__ import annotations

import re
import socket

from functools import lru_cache
from typing import (
    Optional,
    Union,
)

from slixmpp.stringprep import nodeprep, resourceprep, idna, StringprepError

HAVE_INET_PTON = hasattr(socket, 'inet_pton')

#: The basic regex pattern that a JID must match in order to determine
#: the local, domain, and resource parts. This regex does NOT do any
#: validation, which requires application of nodeprep, resourceprep, etc.
JID_PATTERN = re.compile(
    "^(?:([^\"&'/:<>@]{1,1023})@)?([^/@]{1,1023})(?:/(.{1,1023}))?$"
)

#: The set of escape sequences for the characters not allowed by nodeprep.
JID_ESCAPE_SEQUENCES = {'\\20', '\\22', '\\26', '\\27', '\\2f',
                        '\\3a', '\\3c', '\\3e', '\\40', '\\5c'}

#: The reverse mapping of escape sequences to their original forms.
JID_UNESCAPE_TRANSFORMATIONS = {'\\20': ' ',
                                '\\22': '"',
                                '\\26': '&',
                                '\\27': "'",
                                '\\2f': '/',
                                '\\3a': ':',
                                '\\3c': '<',
                                '\\3e': '>',
                                '\\40': '@',
                                '\\5c': '\\'}


# TODO: Find the best cache size for a standard usage.
@lru_cache(maxsize=1024)
def _parse_jid(data: str):
    """
    Parse string data into the node, domain, and resource
    components of a JID, if possible.

    :param string data: A string that is potentially a JID.

    :raises InvalidJID:

    :returns: tuple of the validated local, domain, and resource strings
    """
    match = JID_PATTERN.match(data)
    if not match:
        raise InvalidJID('JID could not be parsed')

    (node, domain, resource) = match.groups()

    node = _validate_node(node)
    domain = _validate_domain(domain)
    resource = _validate_resource(resource)

    return node, domain, resource


def _validate_node(node: Optional[str]):
    """Validate the local, or username, portion of a JID.

    :raises InvalidJID:

    :returns: The local portion of a JID, as validated by nodeprep.
    """
    if node is None:
        return ''

    try:
        node = nodeprep(node)
    except StringprepError:
        raise InvalidJID('Nodeprep failed')

    if not node:
        raise InvalidJID('Localpart must not be 0 bytes')
    if len(node) > 1023:
        raise InvalidJID('Localpart must be less than 1024 bytes')
    return node


def _validate_domain(domain: str):
    """Validate the domain portion of a JID.

    IP literal addresses are left as-is, if valid. Domain names
    are stripped of any trailing label separators (`.`), and are
    checked with the nameprep profile of stringprep. If the given
    domain is actually a punyencoded version of a domain name, it
    is converted back into its original Unicode form. Domains must
    also not start or end with a dash (`-`).

    :raises InvalidJID:

    :returns: The validated domain name
    """
    ip_addr = False

    # First, check if this is an IPv4 address
    try:
        socket.inet_aton(domain)
        ip_addr = True
    except socket.error:
        pass

    # Check if this is an IPv6 address
    if not ip_addr and HAVE_INET_PTON and domain[0] == '[' and domain[-1] == ']':
        try:
            ip = domain[1:-1]
            socket.inet_pton(socket.AF_INET6, ip)
            ip_addr = True
        except (socket.error, ValueError):
            pass

    if not ip_addr:
        # This is a domain name, which must be checked further

        if domain and domain[-1] == '.':
            domain = domain[:-1]

        try:
            domain = idna(domain)
        except StringprepError:
            raise InvalidJID(f'idna validation failed: {domain}')

        if ':' in domain:
            raise InvalidJID(f'Domain containing a port: {domain}')
        for label in domain.split('.'):
            if not label:
                raise InvalidJID(f'Domain containing too many dots: {domain}')
            if '-' in (label[0], label[-1]):
                raise InvalidJID(f'Domain starting or ending with -: {domain}')

    if not domain:
        raise InvalidJID('Domain must not be 0 bytes')
    if len(domain) > 1023:
        raise InvalidJID('Domain must be less than 1024 bytes')

    return domain


def _validate_resource(resource: Optional[str]):
    """Validate the resource portion of a JID.

    :raises InvalidJID:

    :returns: The local portion of a JID, as validated by resourceprep.
    """
    if resource is None:
        return ''

    try:
        resource = resourceprep(resource)
    except StringprepError:
        raise InvalidJID('Resourceprep failed')

    if not resource:
        raise InvalidJID('Resource must not be 0 bytes')
    if len(resource) > 1023:
        raise InvalidJID('Resource must be less than 1024 bytes')
    return resource


def _unescape_node(node: str):
    """Unescape a local portion of a JID.

    .. note::
        The unescaped local portion is meant ONLY for presentation,
        and should not be used for other purposes.
    """
    unescaped = []
    seq = ''
    for i, char in enumerate(node):
        if char == '\\':
            seq = node[i:i+3]
            if seq not in JID_ESCAPE_SEQUENCES:
                seq = ''
        if seq:
            if len(seq) == 3:
                unescaped.append(JID_UNESCAPE_TRANSFORMATIONS.get(seq, char))

            # Pop character off the escape sequence, and ignore it
            seq = seq[1:]
        else:
            unescaped.append(char)
    return ''.join(unescaped)


def _format_jid(
        local: Optional[str] = None,
        domain: Optional[str] = None,
        resource: Optional[str] = None,
    ):
    """Format the given JID components into a full or bare JID.

    :param string local: Optional. The local portion of the JID.
    :param string domain: Required. The domain name portion of the JID.
    :param strin resource: Optional. The resource portion of the JID.

    :return: A full or bare JID string.
    """
    if domain is None:
        return ''
    if local is not None:
        result = local + '@' + domain
    else:
        result = domain
    if resource is not None:
        result += '/' + resource
    return result


class InvalidJID(ValueError):
    """
    Raised when attempting to create a JID that does not pass validation.

    It can also be raised if modifying an existing JID in such a way as
    to make it invalid, such trying to remove the domain from an existing
    full JID while the local and resource portions still exist.
    """

# pylint: disable=R0903
class UnescapedJID:

    """
    .. versionadded:: 1.1.10
    """

    __slots__ = ('_node', '_domain', '_resource')

    def __init__(
            self,
            node: Optional[str],
            domain: Optional[str],
            resource: Optional[str],
        ):
        self._node = node
        self._domain = domain
        self._resource = resource

    def __getattribute__(self, name: str):
        """Retrieve the given JID component.

        :param name: one of: user, server, domain, resource,
                     full, or bare.
        """
        if name == 'resource':
            return self._resource or ''
        if name in ('user', 'username', 'local', 'node'):
            return self._node or ''
        if name in ('server', 'domain', 'host'):
            return self._domain or ''
        if name in ('full', 'jid'):
            return _format_jid(self._node, self._domain, self._resource)
        if name == 'bare':
            return _format_jid(self._node, self._domain)
        return object.__getattribute__(self, name)

    def __str__(self):
        """Use the full JID as the string value."""
        return _format_jid(self._node, self._domain, self._resource)

    def __repr__(self):
        """Use the full JID as the representation."""
        return _format_jid(self._node, self._domain, self._resource)


class JID:

    """
    A representation of a Jabber ID, or JID.

    Each JID may have three components: a user, a domain, and an optional
    resource. For example: user@domain/resource

    When a resource is not used, the JID is called a bare JID.
    The JID is a full JID otherwise.

    **JID Properties:**
        :full: The string value of the full JID.
        :jid: Alias for ``full``.
        :bare: The string value of the bare JID.
        :node: The node portion of the JID.
        :user: Alias for ``node``.
        :local: Alias for ``node``.
        :username: Alias for ``node``.
        :domain: The domain name portion of the JID.
        :server: Alias for ``domain``.
        :host: Alias for ``domain``.
        :resource: The resource portion of the JID.

    :param string jid:
        A string of the form ``'[user@]domain[/resource]'``.
    :param bool bare:
        If present, discard the provided resource.

    :raises InvalidJID:
    """

    __slots__ = ('_node', '_domain', '_resource', '_bare', '_full')

    def __init__(self, jid: Optional[Union[str, 'JID']] = None, bare: bool = False):
        if not jid:
            self._node = ''
            self._domain = ''
            self._resource = ''
            self._bare = ''
            self._full = ''
            return
        elif not isinstance(jid, JID):
            node, domain, resource = _parse_jid(jid)
            self._node = node
            self._domain = domain
            self._resource = resource if not bare else ''
        else:
            self._node = jid._node
            self._domain = jid._domain
            self._resource = jid._resource if not bare else ''
        self._update_bare_full()

    def unescape(self):
        """Return an unescaped JID object.

        Using an unescaped JID is preferred for displaying JIDs
        to humans, and they should NOT be used for any other
        purposes than for presentation.

        :return: :class:`UnescapedJID`

        .. versionadded:: 1.1.10
        """
        return UnescapedJID(_unescape_node(self._node),
                            self._domain,
                            self._resource)

    def _update_bare_full(self):
        """Format the given JID into a bare and a full JID.
        """
        self._bare = (self._node + '@' + self._domain
                      if self._node
                      else self._domain)
        self._full = (self._bare + '/' + self._resource
                      if self._resource
                      else self._bare)

    @property
    def bare(self) -> str:
        return self._bare

    @bare.setter
    def bare(self, value: str):
        node, domain, resource = _parse_jid(value)
        assert not resource
        self._node = node
        self._domain = domain
        self._update_bare_full()


    @property
    def node(self) -> str:
        return self._node

    @node.setter
    def node(self, value: Optional[str]):
        self._node = _validate_node(value)
        self._update_bare_full()

    @property
    def domain(self) -> str:
        return self._domain

    @domain.setter
    def domain(self, value: str):
        self._domain = _validate_domain(value)
        self._update_bare_full()

    @property
    def resource(self) -> str:
        return self._resource

    @resource.setter
    def resource(self, value: Optional[str]):
        self._resource = _validate_resource(value)
        self._update_bare_full()

    @property
    def full(self) -> str:
        return self._full

    @full.setter
    def full(self, value: str):
        self._node, self._domain, self._resource = _parse_jid(value)
        self._update_bare_full()

    user = node
    local = node
    username = node

    server = domain
    host = domain

    jid = full

    def __str__(self):
        """Use the full JID as the string value."""
        return self._full

    def __repr__(self):
        """Use the full JID as the representation."""
        return self._full

    # pylint: disable=W0212
    def __eq__(self, other):
        """Two JIDs are equal if they have the same full JID value."""
        if isinstance(other, UnescapedJID):
            return False
        if not isinstance(other, JID):
            try:
                other = JID(other)
            except InvalidJID:
                return NotImplemented

        return (self._node == other._node and
                self._domain == other._domain and
                self._resource == other._resource)

    def __ne__(self, other):
        """Two JIDs are considered unequal if they are not equal."""
        return not self == other

    def __hash__(self):
        """Hash a JID based on the string version of its full JID."""
        return hash(self._full)



# Slixmpp: The Slick XMPP Library
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.
from slixmpp.plugins import register_plugin
from slixmpp.plugins.xep_0317 import stanza
from slixmpp.plugins.xep_0317.hats import XEP_0317
from slixmpp.plugins.xep_0317.stanza import Hat, Hats

register_plugin(XEP_0317)

__all__ = ['stanza', 'XEP_317']



import unittest
from slixmpp import Iq
from slixmpp.test import SlixTest
import slixmpp.plugins.gmail_notify as gmail
from slixmpp.xmlstream import register_stanza_plugin, ET


class TestGmail(SlixTest):

    def setUp(self):
        register_stanza_plugin(Iq, gmail.GmailQuery)
        register_stanza_plugin(Iq, gmail.MailBox)
        register_stanza_plugin(Iq, gmail.NewMail)

    def testCreateQuery(self):
        """Testing querying Gmail for emails."""

        iq = self.Iq()
        iq['type'] = 'get'
        iq['gmail']['search'] = 'is:starred'
        iq['gmail']['newer-than-time'] = '1140638252542'
        iq['gmail']['newer-than-tid'] = '11134623426430234'

        self.check(iq, """
          <iq type="get">
            <query xmlns="google:mail:notify"
                   newer-than-time="1140638252542"
                   newer-than-tid="11134623426430234"
                   q="is:starred" />
          </iq>
        """, use_values=False)

    def testMailBox(self):
        """Testing reading from Gmail mailbox result"""

        # Use the example from Google's documentation at
        # http://code.google.com/apis/talk/jep_extensions/gmail.html#notifications
        xml = ET.fromstring("""
          <iq type="result">
            <mailbox xmlns="google:mail:notify"
                     result-time='1118012394209'
                     url='http://mail.google.com/mail'
                     total-matched='95'
                     total-estimate='0'>
              <mail-thread-info tid='1172320964060972012'
                                participation='1'
                                messages='28'
                                date='1118012394209'
                                url='http://mail.google.com/mail?view=cv'>
                <senders>
                  <sender name='Me' address='romeo@gmail.com' originator='1' />
                  <sender name='Benvolio' address='benvolio@gmail.com' />
                  <sender name='Mercutio' address='mercutio@gmail.com' unread='1'/>
                </senders>
                <labels>act1scene3</labels>
                <subject>Put thy rapier up.</subject>
                <snippet>Ay, ay, a scratch, a scratch; marry, 'tis enough.</snippet>
              </mail-thread-info>
            </mailbox>
          </iq>
        """)

        iq = self.Iq(xml=xml)
        mailbox = iq['mailbox']
        self.assertTrue(mailbox['result-time'] == '1118012394209', "result-time doesn't match")
        self.assertTrue(mailbox['url'] == 'http://mail.google.com/mail', "url doesn't match")
        self.assertTrue(mailbox['matched'] == '95', "total-matched incorrect")
        self.assertTrue(mailbox['estimate'] == False, "total-estimate incorrect")
        self.assertTrue(len(mailbox['threads']) == 1, "could not extract message threads")

        thread = mailbox['threads'][0]
        self.assertTrue(thread['tid'] == '1172320964060972012', "thread tid doesn't match")
        self.assertTrue(thread['participation'] == '1', "thread participation incorrect")
        self.assertTrue(thread['messages'] == '28', "thread message count incorrect")
        self.assertTrue(thread['date'] == '1118012394209', "thread date doesn't match")
        self.assertTrue(thread['url'] == 'http://mail.google.com/mail?view=cv', "thread url doesn't match")
        self.assertTrue(thread['labels'] == 'act1scene3', "thread labels incorrect")
        self.assertTrue(thread['subject'] == 'Put thy rapier up.', "thread subject doesn't match")
        self.assertTrue(thread['snippet'] == "Ay, ay, a scratch, a scratch; marry, 'tis enough.", "snippet doesn't match")
        self.assertTrue(len(thread['senders']) == 3, "could not extract senders")

        sender1 = thread['senders'][0]
        self.assertTrue(sender1['name'] == 'Me', "sender name doesn't match")
        self.assertTrue(sender1['address'] == 'romeo@gmail.com', "sender address doesn't match")
        self.assertTrue(sender1['originator'] == True, "sender originator incorrect")
        self.assertTrue(sender1['unread'] == False, "sender unread incorrectly True")

        sender2 = thread['senders'][2]
        self.assertTrue(sender2['unread'] == True, "sender unread incorrectly False")

suite = unittest.TestLoader().loadTestsFromTestCase(TestGmail)



import time
import threading

import unittest
from slixmpp.test import SlixTest
from slixmpp.exceptions import IqTimeout
from slixmpp import Callback, MatchXPath


class TestHandlers(SlixTest):
    """
    Test using handlers and waiters.
    """

    def setUp(self):
        self.stream_start()

    def testCallback(self):
        """Test using stream callback handlers."""

        def callback_handler(stanza):
            self.xmpp.send_raw("""
              <message>
                <body>Success!</body>
              </message>
            """)

        callback = Callback('Test Callback',
                            MatchXPath('{test}tester'),
                            callback_handler)

        self.xmpp.register_handler(callback)

        self.recv("""<tester xmlns="test" />""")

        msg = self.Message()
        msg['body'] = 'Success!'
        self.send(msg)

    def testWaiter(self):
        """Test using stream waiter handler."""

        def waiter_handler(stanza):
            iq = self.xmpp.Iq()
            iq['id'] = 'test'
            iq['type'] = 'set'
            iq['query'] = 'test'
            def callback_waiter(result):
                self.xmpp.send_raw("""
                  <message>
                    <body>Successful: %s</body>
                  </message>
                """ % result['query'])
            iq.send(callback=callback_waiter)

        self.xmpp.add_event_handler('message', waiter_handler)

        # Send message to trigger waiter_handler
        self.recv("""
          <message>
            <body>Testing</body>
          </message>
        """)

        # Check that Iq was sent by waiter_handler
        iq = self.Iq()
        iq['id'] = 'test'
        iq['type'] = 'set'
        iq['query'] = 'test'
        self.send(iq)

        # Send the reply Iq
        self.recv("""
          <iq id="test" type="result">
            <query xmlns="test" />
          </iq>
        """)

        # Check that waiter_handler received the reply
        msg = self.Message()
        msg['body'] = 'Successful: test'
        self.send(msg)

    def testWaiterTimeout(self):
        """Test that waiter handler is removed after timeout."""

        def waiter_handler(stanza):
            iq = self.xmpp.Iq()
            iq['id'] = 'test2'
            iq['type'] = 'set'
            iq['query'] = 'test2'
            try:
                reply = iq.send(timeout=0)
            except IqTimeout:
                pass

        self.xmpp.add_event_handler('message', waiter_handler)

        # Start test by triggerig waiter_handler
        self.recv("""<message><body>Start Test</body></message>""")

        # Check that Iq was sent to trigger start of timeout period
        iq = self.Iq()
        iq['id'] = 'test2'
        iq['type'] = 'set'
        iq['query'] = 'test2'
        self.send(iq)

        # Check that the waiter is no longer registered
        waiter_exists = self.xmpp.remove_handler('IqWait_test2')

        self.assertTrue(waiter_exists == False,
            "Waiter handler was not removed.")

    def testIqCallback(self):
        """Test that iq.send(callback=handle_foo) works."""
        events = []

        def handle_foo(iq):
            events.append('foo')

        iq = self.Iq()
        iq['type'] = 'get'
        iq['id'] = 'test-foo'
        iq['to'] = 'user@localhost'
        iq['query'] = 'foo'
        iq.send(callback=handle_foo)

        self.send("""
          <iq type="get" id="test-foo" to="user@localhost">
            <query xmlns="foo" />
          </iq>
        """)

        self.recv("""
          <iq type="result" id="test-foo"
              to="test@localhost"
              from="user@localhost">
            <query xmlns="foo">
              <data />
            </query>
          </iq>
        """)

        self.assertTrue(events == ['foo'],
                "Iq callback was not executed: %s" % events)

    def testMultipleHandlersForStanza(self):
        """
        Test that multiple handlers for a single stanza work
        without clobbering each other.
        """

        def handler_1(msg):
            msg.reply("Handler 1: %s" % msg['body']).send()

        def handler_2(msg):
            msg.reply("Handler 2: %s" % msg['body']).send()

        def handler_3(msg):
            msg.reply("Handler 3: %s" % msg['body']).send()

        self.xmpp.add_event_handler('message', handler_1)
        self.xmpp.add_event_handler('message', handler_2)
        self.xmpp.add_event_handler('message', handler_3)

        self.recv("""
          <message to="tester@localhost" from="user@example.com">
            <body>Testing</body>
          </message>
        """)


        # This test is brittle, depending on the fact that handlers
        # will be checked in the order they are registered.
        self.send("""
          <message to="user@example.com">
            <body>Handler 1: Testing</body>
          </message>
        """)
        self.send("""
          <message to="user@example.com">
            <body>Handler 2: Testing</body>
          </message>
        """)
        self.send("""
          <message to="user@example.com">
            <body>Handler 3: Testing</body>
          </message>
        """)

    def testWrongSender(self):
      """
      Test that using the wrong sender JID in a IQ result
      doesn't trigger handlers.
      """

      events = []

      def callback(result):
          events.append(result['from'].full)

      iq = self.Iq()
      iq['id'] = 'test'
      iq['to'] = 'tester@slixmpp.com/test'
      iq['type'] = 'set'
      iq['query'] = 'test'
      iq.send(callback=callback)

      self.recv("""
        <iq id="test" from="evil@slixmpp.com/bad" type="result">
          <query xmlns="test" />
        </iq>
      """)
      self.recv("""
        <iq id="test" from="evil2@slixmpp.com" type="result">
          <query xmlns="test" />
        </iq>
      """)
      self.recv("""
        <iq id="test" from="evil.com" type="result">
          <query xmlns="test" />
        </iq>
      """)

      # Now for a good one
      self.recv("""
        <iq id="test" from="tester@slixmpp.com/test" type="result">
          <query xmlns="test" />
        </iq>
      """)

      self.assertEqual(events, ['tester@slixmpp.com/test'], "Did not timeout on bad sender")


suite = unittest.TestLoader().loadTestsFromTestCase(TestHandlers)



# Slixmpp: The Slick XMPP Library
# This file is part of Slixmpp
# See the file LICENSE for copying permission
import uuid
import logging

from typing import (
    Optional,
    Union,
)

from slixmpp import JID
from slixmpp.stanza import Message, Iq
from slixmpp.exceptions import XMPPError
from slixmpp.xmlstream.handler import CoroutineCallback
from slixmpp.xmlstream.matcher import StanzaPath
from slixmpp.xmlstream import register_stanza_plugin
from slixmpp.plugins import BasePlugin
from slixmpp.plugins.xep_0047 import stanza, Open, Close, Data, IBBytestream


log = logging.getLogger(__name__)


class XEP_0047(BasePlugin):
    """
    XEP-0047: In-Band Bytestreams

    Events registered by this plugin:

    - :term:`ibb_stream_start`
    - :term:`ibb_stream_end`
    - :term:`ibb_stream_data`
    - :term:`stream:[stream id]:[peer jid]`

    Plugin Parameters:

    - ``block_size`` (default: ``4096``): default block size to negociate
    - ``max_block_size`` (default: ``8192``): max block size to accept
    - ``auto_accept`` (default: ``False``): if incoming streams should be
        accepted automatically.

    - :term:`authorized (0047 version)`
    - :term:`authorized_sid (0047 version)`
    - :term:`preauthorize_sid (0047 version)`
    - :term:`get_stream`
    - :term:`set_stream`
    - :term:`del_stream`
    """

    name = 'xep_0047'
    description = 'XEP-0047: In-Band Bytestreams'
    dependencies = {'xep_0030'}
    stanza = stanza
    default_config = {
        'block_size': 4096,
        'max_block_size': 8192,
        'auto_accept': False,
    }

    def plugin_init(self):
        self._streams = {}
        self._preauthed_sids = {}

        register_stanza_plugin(Iq, Open)
        register_stanza_plugin(Iq, Close)
        register_stanza_plugin(Iq, Data)
        register_stanza_plugin(Message, Data)

        self.xmpp.register_handler(CoroutineCallback(
            'IBB Open',
            StanzaPath('iq@type=set/ibb_open'),
            self._handle_open_request))

        self.xmpp.register_handler(CoroutineCallback(
            'IBB Close',
            StanzaPath('iq@type=set/ibb_close'),
            self._handle_close))

        self.xmpp.register_handler(CoroutineCallback(
            'IBB Data',
            StanzaPath('iq@type=set/ibb_data'),
            self._handle_data))

        self.xmpp.register_handler(CoroutineCallback(
            'IBB Message Data',
            StanzaPath('message/ibb_data'),
            self._handle_data))

        self.api.register(self._authorized, 'authorized', default=True)
        self.api.register(self._authorized_sid, 'authorized_sid', default=True)
        self.api.register(self._preauthorize_sid, 'preauthorize_sid', default=True)
        self.api.register(self._get_stream, 'get_stream', default=True)
        self.api.register(self._set_stream, 'set_stream', default=True)
        self.api.register(self._del_stream, 'del_stream', default=True)

    def plugin_end(self):
        self.xmpp.remove_handler('IBB Open')
        self.xmpp.remove_handler('IBB Close')
        self.xmpp.remove_handler('IBB Data')
        self.xmpp.remove_handler('IBB Message Data')
        self.xmpp['xep_0030'].del_feature(feature='http://jabber.org/protocol/ibb')

    def session_bind(self, jid):
        self.xmpp['xep_0030'].add_feature('http://jabber.org/protocol/ibb')

    def _get_stream(self, jid, sid, peer_jid, data):
        return self._streams.get((jid, sid, peer_jid), None)

    def _set_stream(self, jid, sid, peer_jid, stream):
        self._streams[(jid, sid, peer_jid)] = stream

    def _del_stream(self, jid, sid, peer_jid, data):
        if (jid, sid, peer_jid) in self._streams:
            del self._streams[(jid, sid, peer_jid)]

    async def _accept_stream(self, iq):
        receiver = iq['to']
        sender = iq['from']
        sid = iq['ibb_open']['sid']

        if await self.api['authorized_sid'](receiver, sid, sender, iq):
            return True
        return await self.api['authorized'](receiver, sid, sender, iq)

    def _authorized(self, jid, sid, ifrom, iq):
        if self.auto_accept:
            return True
        return False

    def _authorized_sid(self, jid, sid, ifrom, iq):
        if (jid, sid, ifrom) in self._preauthed_sids:
            del self._preauthed_sids[(jid, sid, ifrom)]
            return True
        return False

    def _preauthorize_sid(self, jid, sid, ifrom, data):
        self._preauthed_sids[(jid, sid, ifrom)] = True

    async def open_stream(self, jid: JID, *, block_size: Optional[int] = None,
                          sid: Optional[str] = None, use_messages: bool = False,
                          ifrom: Optional[JID] = None,
                          **iqkwargs) -> IBBytestream:
        """Open an IBB stream with a peer JID.

        .. versionchanged:: 1.8.0
            This function is now a coroutine and must be awaited.
            All parameters except ``jid`` are keyword-args only.

        :param jid: The remote JID to initiate the stream with.
        :param block_size: The block size to advertise.
        :param sid: The IBB stream id (if not provided, will be auto-generated).
        :param use_messages: If the stream should use message stanzas instead of iqs.
        :returns: The opened byte stream with the remote JID
        :raises .IqError: When the remote entity denied the stream.
        """
        if sid is None:
            sid = str(uuid.uuid4())
        if block_size is None:
            block_size = self.block_size

        iq = self.xmpp.make_iq_set(ito=jid, ifrom=ifrom)
        iq['ibb_open']['block_size'] = block_size
        iq['ibb_open']['sid'] = sid
        iq['ibb_open']['stanza'] = 'message' if use_messages else 'iq'

        stream = IBBytestream(self.xmpp, sid, block_size,
                              iq['from'], iq['to'], use_messages)

        callback = iqkwargs.pop('callback', None)
        result = await iq.send(**iqkwargs)

        log.debug('IBB stream (%s) accepted by %s', stream.sid, result['from'])
        stream.self_jid = result['to']
        stream.peer_jid = result['from']
        stream.stream_started = True
        await self.api['set_stream'](stream.self_jid, stream.sid, stream.peer_jid, stream)
        if callback is not None:
            self.xmpp.add_event_handler('ibb_stream_start', callback, disposable=True)
        self.xmpp.event('ibb_stream_start', stream)
        self.xmpp.event('stream:%s:%s' % (stream.sid, stream.peer_jid), stream)
        return stream

    async def _handle_open_request(self, iq: Iq):
        sid = iq['ibb_open']['sid']
        size = iq['ibb_open']['block_size'] or self.block_size

        log.debug('Received IBB stream request from %s', iq['from'])

        if not sid:
            raise XMPPError(etype='modify', condition='bad-request')

        if not await self._accept_stream(iq):
            raise XMPPError(etype='cancel', condition='not-acceptable')

        if size > self.max_block_size:
            raise XMPPError('resource-constraint')

        stream = IBBytestream(self.xmpp, sid, size,
                              iq['to'], iq['from'])
        stream.stream_started = True
        await self.api['set_stream'](stream.self_jid, stream.sid, stream.peer_jid, stream)
        iq.reply().send()

        self.xmpp.event('ibb_stream_start', stream)
        self.xmpp.event('stream:%s:%s' % (sid, stream.peer_jid), stream)

    async def _handle_data(self, stanza: Union[Iq, Message]):
        sid = stanza['ibb_data']['sid']
        stream = await self.api['get_stream'](stanza['to'], sid, stanza['from'])
        if stream is not None and stanza['from'] == stream.peer_jid:
            stream._recv_data(stanza)
        else:
            raise XMPPError('item-not-found')

    async def _handle_close(self, iq: Iq):
        sid = iq['ibb_close']['sid']
        stream = await self.api['get_stream'](iq['to'], sid, iq['from'])
        if stream is not None and iq['from'] == stream.peer_jid:
            stream._closed(iq)
            await self.api['del_stream'](stream.self_jid, stream.sid, stream.peer_jid)
        else:
            raise XMPPError('item-not-found')



# Slixmpp: The Slick XMPP Library
# Copyright (C) 2010  Nathanael C. Fritz
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.
import logging

from xml.parsers.expat import ExpatError
from xml.etree.ElementTree import Element

from slixmpp.xmlstream.stanzabase import ET, StanzaBase
from slixmpp.xmlstream.matcher.base import MatcherBase


log = logging.getLogger(__name__)


class MatchXMLMask(MatcherBase):

    """
    The XMLMask matcher selects stanzas whose XML matches a given
    XML pattern, or mask. For example, message stanzas with body elements
    could be matched using the mask:

    .. code-block:: xml

        <message xmlns="jabber:client"><body /></message>

    Use of XMLMask is discouraged, and
    :class:`~slixmpp.xmlstream.matcher.xpath.MatchXPath` or
    :class:`~slixmpp.xmlstream.matcher.stanzapath.StanzaPath`
    should be used instead.

    :param criteria: Either an :class:`~xml.etree.ElementTree.Element` XML
                     object or XML string to use as a mask.
    """
    _criteria: Element

    def __init__(self, criteria: str, default_ns: str = 'jabber:client'):
        MatcherBase.__init__(self, criteria)
        if isinstance(criteria, str):
            self._criteria = ET.fromstring(criteria)
        self.default_ns = default_ns

    def setDefaultNS(self, ns: str) -> None:
        """Set the default namespace to use during comparisons.

        :param ns: The new namespace to use as the default.
        """
        self.default_ns = ns

    def match(self, xml: StanzaBase) -> bool:
        """Compare a stanza object or XML object against the stored XML mask.

        Overrides MatcherBase.match.

        :param xml: The stanza object or XML object to compare against.
        """
        real_xml = xml.xml
        return self._mask_cmp(real_xml, self._criteria, True)

    def _mask_cmp(self, source: Element, mask: Element, use_ns: bool = False,
                  default_ns: str = '__no_ns__') -> bool:
        """Compare an XML object against an XML mask.

        :param source: The :class:`~xml.etree.ElementTree.Element` XML object
                       to compare against the mask.
        :param mask: The :class:`~xml.etree.ElementTree.Element` XML object
                     serving as the mask.
        :param use_ns: Indicates if namespaces should be respected during
                       the comparison.
        :default_ns: The default namespace to apply to elements that
                     do not have a specified namespace.
                     Defaults to ``"__no_ns__"``.
        """
        if source is None:
            # If the element was not found. May happen during recursive calls.
            return False

        mask_ns_tag = "{%s}%s" % (self.default_ns, mask.tag)
        if source.tag not in [mask.tag, mask_ns_tag]:
            return False

        # If the mask includes text, compare it.
        if mask.text and source.text and \
           source.text.strip() != mask.text.strip():
            return False

        # Compare attributes. The stanza must include the attributes
        # defined by the mask, but may include others.
        for name, value in mask.attrib.items():
            if source.attrib.get(name, "__None__") != value:
                return False

        # Recursively check subelements.
        matched_elements = {}
        for subelement in mask:
            matched = False
            for other in source.findall(subelement.tag):
                matched_elements[other] = False
                if self._mask_cmp(other, subelement, use_ns):
                    if not matched_elements.get(other, False):
                        matched_elements[other] = True
                        matched = True
            if not matched:
                return False

        # Everything matches.
        return True



# Slixmpp: The Slick XMPP Library
# Copyright (C) 2011  Nathanael C. Fritz
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.
import ssl
import logging

from slixmpp.util import sasl
from slixmpp.util.stringprep_profiles import StringPrepError
from slixmpp.stanza import StreamFeatures
from slixmpp.xmlstream import register_stanza_plugin
from slixmpp.plugins import BasePlugin
from slixmpp.xmlstream.matcher import MatchXPath
from slixmpp.xmlstream.handler import Callback
from slixmpp.features.feature_mechanisms import stanza

from typing import ClassVar, Set


log = logging.getLogger(__name__)


class FeatureMechanisms(BasePlugin):

    name = 'feature_mechanisms'
    description = 'RFC 6120: Stream Feature: SASL'
    dependencies: ClassVar[Set[str]] = set()
    stanza = stanza
    default_config = {
        'use_mech': None,
        'use_mechs': None,
        'min_mech': None,
        'sasl_callback': None,
        'security_callback': None,
        'encrypted_plain': True,
        'unencrypted_plain': False,
        'unencrypted_digest': False,
        'unencrypted_cram': False,
        'unencrypted_scram': True,
        'order': 100,
        'tls_version': None,
    }

    def plugin_init(self):
        if self.sasl_callback is None:
            self.sasl_callback = self._default_credentials

        if self.security_callback is None:
            self.security_callback = self._default_security

        creds = self.sasl_callback({'username'}, set())
        if not self.use_mech and not creds['username']:
            self.use_mech = 'ANONYMOUS'

        self.mech = None
        self.mech_list = set()
        self.attempted_mechs = set()

        register_stanza_plugin(StreamFeatures, stanza.Mechanisms)

        self.xmpp.register_stanza(stanza.Success)
        self.xmpp.register_stanza(stanza.Failure)
        self.xmpp.register_stanza(stanza.Auth)
        self.xmpp.register_stanza(stanza.Challenge)
        self.xmpp.register_stanza(stanza.Response)
        self.xmpp.register_stanza(stanza.Abort)

        self.xmpp.register_handler(
                Callback('SASL Success',
                         MatchXPath(stanza.Success.tag_name()),
                         self._handle_success,
                         instream=True))
        self.xmpp.register_handler(
                Callback('SASL Failure',
                         MatchXPath(stanza.Failure.tag_name()),
                         self._handle_fail,
                         instream=True))
        self.xmpp.register_handler(
                Callback('SASL Challenge',
                         MatchXPath(stanza.Challenge.tag_name()),
                         self._handle_challenge))

        self.xmpp.register_feature('mechanisms',
                self._handle_sasl_auth,
                restart=True,
                order=self.order)

    def _default_credentials(self, required_values, optional_values):
        creds = self.xmpp.credentials
        result = {}
        values = required_values.union(optional_values)
        for value in values:
            if value == 'username':
                result[value] = creds.get('username', self.xmpp.requested_jid.user)
            elif value == 'email':
                jid = self.xmpp.requested_jid.bare
                result[value] = creds.get('email', jid)
            elif value == 'channel_binding':
                if isinstance(self.xmpp.socket, (ssl.SSLSocket, ssl.SSLObject)):
                    version = self.xmpp.socket.version()
                    # As of now, python does not implement anything else
                    # than tls-unique, which is forbidden on TLSv1.3
                    # see https://github.com/python/cpython/issues/95341
                    if version != 'TLSv1.3':
                        result[value] = self.xmpp.socket.get_channel_binding(
                            cb_type="tls-unique"
                        )
                    elif 'tls-exporter' in ssl.CHANNEL_BINDING_TYPES:
                        result[value] = self.xmpp.socket.get_channel_binding(
                            cb_type="tls-exporter"
                        )
                    else:
                        result[value] = None
                else:
                    result[value] = None
            elif value == 'host':
                result[value] = creds.get('host', self.xmpp.requested_jid.domain)
            elif value == 'realm':
                result[value] = creds.get('realm', self.xmpp.requested_jid.domain)
            elif value == 'service-name':
                result[value] = creds.get('service-name', self.xmpp._service_name)
            elif value == 'service':
                result[value] = creds.get('service', 'xmpp')
            elif value in creds:
                result[value] = creds[value]
        return result

    def _default_security(self, values):
        result = {}
        for value in values:
            if value == 'encrypted':
                if 'starttls' in self.xmpp.features:
                    result[value] = True
                elif isinstance(self.xmpp.socket, (ssl.SSLSocket, ssl.SSLObject)):
                    result[value] = True
                else:
                    result[value] = False
            elif value == 'tls_version':
                if isinstance(self.xmpp.socket, (ssl.SSLSocket, ssl.SSLObject)):
                    result[value] = self.xmpp.socket.version()
            elif value == 'binding_proposed':
                result[value] = any(x for x in self.mech_list if x.endswith('-PLUS'))
            else:
                result[value] = self.config.get(value, False)
        return result

    def _handle_sasl_auth(self, features):
        """
        Handle authenticating using SASL.

        Arguments:
            features -- The stream features stanza.
        """
        if 'mechanisms' in self.xmpp.features:
            # SASL authentication has already succeeded, but the
            # server has incorrectly offered it again.
            return False

        enforce_limit = False
        limited_mechs = self.use_mechs

        if limited_mechs is None:
            limited_mechs = set()
        elif limited_mechs and not isinstance(limited_mechs, set):
            limited_mechs = set(limited_mechs)
            enforce_limit = True

        if self.use_mech:
            limited_mechs.add(self.use_mech)
            enforce_limit = True

        if enforce_limit:
            self.use_mechs = limited_mechs

        self.mech_list = set(features['mechanisms'])

        return self._send_auth()

    def _send_auth(self):
        mech_list = self.mech_list - self.attempted_mechs
        try:
            self.mech = sasl.choose(mech_list,
                                    self.sasl_callback,
                                    self.security_callback,
                                    limit=self.use_mechs,
                                    min_mech=self.min_mech)
        except sasl.SASLNoAppropriateMechanism:
            log.error("No appropriate login method.")
            self.xmpp.event("failed_all_auth")
            if not self.attempted_mechs:
                # Only trigger this event if we didn't try at least one
                # method
                self.xmpp.event("no_auth")
            self.attempted_mechs = set()
            return self.xmpp.disconnect()
        except StringPrepError:
            log.exception("A credential value did not pass SASLprep.")
            self.xmpp.disconnect()

        resp = stanza.Auth(self.xmpp)
        resp['mechanism'] = self.mech.name
        try:
            resp['value'] = self.mech.process()
        except sasl.SASLCancelled:
            self.attempted_mechs.add(self.mech.name)
            self._send_auth()
        except sasl.SASLMutualAuthFailed:
            log.error("Mutual authentication failed! " + \
                      "A security breach is possible.")
            self.attempted_mechs.add(self.mech.name)
            self.xmpp.disconnect()
        except sasl.SASLFailed:
            self.attempted_mechs.add(self.mech.name)
            self._send_auth()
        else:
            resp.send()

        return True

    def _handle_challenge(self, stanza):
        """SASL challenge received. Process and send response."""
        resp = self.stanza.Response(self.xmpp)
        try:
            resp['value'] = self.mech.process(stanza['value'])
        except sasl.SASLCancelled:
            self.stanza.Abort(self.xmpp).send()
        except sasl.SASLMutualAuthFailed:
            log.error("Mutual authentication failed! " + \
                      "A security breach is possible.")
            self.attempted_mechs.add(self.mech.name)
            self.xmpp.disconnect()
        except sasl.SASLFailed:
            self.stanza.Abort(self.xmpp).send()
        else:
            if resp.get_value() == '':
                resp.del_value()
            resp.send()

    def _handle_success(self, stanza):
        """SASL authentication succeeded. Restart the stream."""
        try:
            final = self.mech.process(stanza['value'])
        except sasl.SASLMutualAuthFailed:
            log.error("Mutual authentication failed! " + \
                      "A security breach is possible.")
            self.attempted_mechs.add(self.mech.name)
            self.xmpp.disconnect()
        else:
            self.attempted_mechs = set()
            self.xmpp.authenticated = True
            self.xmpp.features.add('mechanisms')
            self.xmpp.event('auth_success', stanza)
            # Restart the stream
            self.xmpp.init_parser()
            self.xmpp.send_raw(self.xmpp.stream_header)

    def _handle_fail(self, stanza):
        """SASL authentication failed. Disconnect and shutdown."""
        self.attempted_mechs.add(self.mech.name)
        log.info("Authentication failed: %s", stanza['condition'])
        self.xmpp.event("failed_auth", stanza)
        self._send_auth()
        return True



import logging

from slixmpp import Message, register_stanza_plugin
from slixmpp.plugins import BasePlugin

from . import stanza

log = logging.getLogger(__name__)


class XEP_0372(BasePlugin):
    """
    XEP-0372: References

    Minimum needed for xep 0385 (Stateless inline media sharing)
    """

    name = "xep_0372"
    description = "XEP-0372: References"
    stanza = stanza

    def plugin_init(self):
        register_stanza_plugin(Message, stanza.Reference)




# Slixmpp: The Slick XMPP Library
# Copyright (C) 2010  Nathanael C. Fritz
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.
from slixmpp import JID
from slixmpp.stanza import Presence
from slixmpp.roster import RosterItem
from slixmpp.types import RosterState, RosterDBProtocol, JidStr, ResourceDict

from typing import TYPE_CHECKING, Optional, Dict, List, Union, Iterator

if TYPE_CHECKING:
    from slixmpp import BaseXMPP


class RosterNode:

    """
    A roster node is a roster for a single JID.

    Attributes:
        xmpp           -- The main Slixmpp instance.
        jid            -- The JID that owns the roster node.
        db             -- Optional interface to an external datastore.
        auto_authorize -- Determines how authorizations are handled:
                            True  -- Accept all subscriptions.
                            False -- Reject all subscriptions.
                            None  -- Subscriptions must be
                                     manually authorized.
                          Defaults to True.
        auto_subscribe -- Determines if bi-directional subscriptions
                          are created after automatically authrorizing
                          a subscription request.
                          Defaults to True
        last_status    -- The last sent presence status that was broadcast
                          to all contact JIDs.

    Methods:
        add           -- Add a JID to the roster.
        update        -- Update a JID's subscription information.
        subscribe     -- Subscribe to a JID.
        unsubscribe   -- Unsubscribe from a JID.
        remove        -- Remove a JID from the roster.
        presence      -- Return presence information for a JID's resources.
        send_presence -- Shortcut for sending a presence stanza.
    """
    xmpp: "BaseXMPP"
    jid: JidStr
    ignore_updates: bool
    auto_authorize: bool
    auto_subscribe: bool
    _jids: dict[str, RosterItem]
    db: Optional[RosterDBProtocol]
    last_status: Optional[Presence]

    def __init__(self, xmpp: "BaseXMPP", jid: JidStr, db: Optional[RosterDBProtocol] = None) -> None:
        """
        Create a roster node for a JID.

        :param xmpp: The main Slixmpp instance.
        :param jid: The JID that owns the roster.
        :param db: Optional interface to an external datastore.
        """
        self.xmpp = xmpp
        self.jid = jid
        self.db = db
        self.ignore_updates = False
        self.auto_authorize = True
        self.auto_subscribe = True
        self.last_status = None
        self._version = ''
        self._jids = {}

        if self.db:
            if hasattr(self.db, 'version'):
                self._version = self.db.version(self.jid)
            for jid in self.db.entries(self.jid):
                self.add(jid)

    @property
    def version(self) -> str:
        """Retrieve the roster's version ID."""
        if self.db and hasattr(self.db, 'version'):
            self._version = self.db.version(self.jid)
        return self._version

    @version.setter
    def version(self, version: str) -> None:
        """Set the roster's version ID."""
        self._version = version
        if self.db and hasattr(self.db, 'set_version'):
            self.db.set_version(self.jid, version)

    def __getitem__(self, key: JidStr) -> RosterItem:
        """
        Return the roster item for a subscribed JID.

        A new item entry will be created if one does not already exist.
        """
        if key is None:
            key = JID('')
        if not isinstance(key, JID):
            key = JID(key)
        key = key.bare
        if key not in self._jids:
            self.add(key, save=True)
        return self._jids[key]

    def __delitem__(self, key: JidStr) -> None:
        """
        Remove a roster item from the local storage.

        To remove an item from the server, use the remove() method.
        """
        if key is None:
            key = JID('')
        if not isinstance(key, JID):
            key = JID(key)
        key = key.bare
        if key in self._jids:
            del self._jids[key]

    def __len__(self) -> int:
        """Return the number of JIDs referenced by the roster."""
        return len(self._jids)

    def keys(self) -> List[str]:
        """Return a list of all subscribed JIDs."""
        return list(self._jids.keys())

    def has_jid(self, jid: JidStr) -> bool:
        """Returns whether the roster has a JID."""
        return jid in self._jids

    def groups(self) -> Dict[str, List[str]]:
        """Return a dictionary mapping group names to JIDs."""
        result: Dict[str, List[str]] = {}
        for jid in self._jids:
            groups = self._jids[jid]['groups']
            if not groups:
                if '' not in result:
                    result[''] = []
                result[''].append(jid)
            for group in groups:
                if group not in result:
                    result[group] = []
                result[group].append(jid)
        return result

    def __iter__(self) -> Iterator[str]:
        """Iterate over the roster items."""
        return self._jids.__iter__()

    def set_backend(self, db: Optional[RosterDBProtocol] = None, save: bool = True) -> None:
        """
        Set the datastore interface object for the roster node.

        :param db: The new datastore interface.
        :param save: If True, save the existing state to the new
                     backend datastore. Defaults to True.
        """
        self.db = db
        if self.db:
            existing_entries = set(self._jids)
            new_entries = set(self.db.entries(self.jid, {}))

            for jid in existing_entries:
                self._jids[jid].set_backend(db, save)
            for jid in new_entries - existing_entries:
                self.add(jid)

    def add(self, jid: JidStr, name: str = '', groups: Optional[List[str]] = None,
            afrom: bool = False, ato: bool = False, pending_in: bool = False,
            pending_out: bool = False, whitelisted: bool = False,
            save: bool = False) -> None:
        """
        Add a new roster item entry.

        :param jid: The JID for the roster item.
        :param name: An alias for the JID.
        :param groups: A list of group names.
        :param afrom: Indicates if the JID has a subscription state
                      of 'from'. Defaults to False.
        :param ato: Indicates if the JID has a subscription state
                    of 'to'. Defaults to False.
        :param pending_in: Indicates if the JID has sent a subscription
                           request to this connection's JID.
                           Defaults to False.
        :param pending_out: Indicates if a subscription request has been sent
                            to this JID.
                            Defaults to False.
        :param whitelisted: Indicates if a subscription request from this JID
                            should be automatically authorized.
                            Defaults to False.
        :param save: Indicates if the item should be persisted
                     immediately to an external datastore,
                     if one is used.
                     Defaults to False.
        """
        if isinstance(jid, JID):
            key = jid.bare
        else:
            key = jid

        state = RosterState({
            'name': name,
            'groups': groups or [],
            'from': afrom,
            'to': ato,
            'pending_in': pending_in,
            'pending_out': pending_out,
            'whitelisted': whitelisted,
            'subscription': 'none',
            'removed': False,
        })
        self._jids[key] = RosterItem(self.xmpp, jid, self.jid,
                                     state=state, db=self.db,
                                     roster=self)
        if save:
            self._jids[key].save()

    def subscribe(self, jid: JidStr) -> None:
        """
        Subscribe to the given JID.

        :param jid: The JID to subscribe to.
        """
        self[jid].subscribe()

    def unsubscribe(self, jid: JidStr) -> None:
        """
        Unsubscribe from the given JID.

        :param jid: The JID to unsubscribe from.
        """
        self[jid].unsubscribe()

    def remove(self, jid: JidStr) -> None:
        """
        Remove a JID from the roster.

        :param jid: The JID to remove.
        """
        self[jid].remove()
        if not self.xmpp.is_component:
            return self.update(jid, subscription='remove')

    def update(self, jid: JidStr, name: Optional[str] = None, subscription=None, groups: Optional[List[str]] = None,
               timeout: Optional[int] = None, callback=None, timeout_callback=None):
        """
        Update a JID's subscription information.

        :param jid: The JID to update.
        :param name: Optional alias for the JID.
        :param subscription: The subscription state. May be one of: 'to',
                             'from', 'both', 'none', or 'remove'.
        :param groups: A list of group names.
        :param timeout: The length of time (in seconds) to wait
                        for a response before continuing if blocking
                        is used. Defaults to self.response_timeout.
        :param callback: Optional reference to a stream handler function.
                         Will be executed when the roster is received.
        """
        if groups is None:
            groups = []

        if name:
            self[jid]['name'] = name
        self[jid]['groups'] = groups
        self[jid].save()

        if not self.xmpp.is_component:
            iq = self.xmpp.Iq()
            iq['type'] = 'set'
            iq['roster']['items'] = {jid: {'name': name,
                                           'subscription': subscription,
                                           'groups': groups}}

            return iq.send(timeout=timeout, callback=callback,
                           timeout_callback=timeout_callback)

    def presence(self, jid: JID, resource: Optional[str] = None) -> Union[ResourceDict, Dict[str, ResourceDict]]:
        """
        Retrieve the presence information of a JID.

        May return either all online resources' status, or
        a single resource's status.

        :param jid: The JID to lookup.
        :param resource: Optional resource for returning
                         only the status of a single connection.
        """
        if resource is None:
            return self[jid].resources

        default_presence = ResourceDict({
            'status': '',
            'priority': 0,
            'show': '',
        })
        return self[jid].resources.get(resource,
                                       default_presence)

    def reset(self) -> None:
        """
        Reset the state of the roster to forget any current
        presence information. Useful after a disconnection occurs.
        """
        for jid in self:
            self[jid].reset()

    def send_presence(self, **kwargs) -> None:
        """
        Create, initialize, and send a Presence stanza.

        If no recipient is specified, send the presence immediately.
        Otherwise, forward the send request to the recipient's roster
        entry for processing.

        Arguments:
        :param pshow: The presence's show value.
        :param pstatus: The presence's status message.
        :param ppriority: This connections' priority.
        :param pto: The recipient of a directed presence.
        :param pfrom: The sender of a directed presence, which should
                      be the owner JID plus resource.
        :param ptype: The type of presence, such as 'subscribe'.
        :param pnick: Optional nickname of the presence's sender.
        """
        if self.xmpp.is_component and not kwargs.get('pfrom', ''):
            kwargs['pfrom'] = self.jid
        self.xmpp.send_presence(**kwargs)

    def send_last_presence(self) -> None:
        """Resend a presence with the last status"""
        if self.last_status is None:
            self.send_presence()
        else:
            pres = self.last_status
            if self.xmpp.is_component:
                pres['from'] = self.jid
            else:
                del pres['from']
            pres.send()

    def __repr__(self) -> str:
        return repr(self._jids)




# Slixmpp: The Slick XMPP Library
# Copyright (C) 2011  Nathanael C. Fritz
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.
from typing import Set, ClassVar
from slixmpp.xmlstream import StanzaBase, ElementBase
from slixmpp.xmlstream.xmlstream import InvalidCABundle

import logging
log = logging.getLogger(__name__)


class STARTTLS(StanzaBase):
    """

    .. code-block:: xml

         <starttls xmlns='urn:ietf:params:xml:ns:xmpp-tls'/>

    """
    name = 'starttls'
    namespace = 'urn:ietf:params:xml:ns:xmpp-tls'
    interfaces = {'required'}
    plugin_attrib = name

    def get_required(self):
        return True


class Proceed(StanzaBase):
    """

    .. code-block:: xml

        <proceed xmlns='urn:ietf:params:xml:ns:xmpp-tls'/>

    """
    name = 'proceed'
    namespace = 'urn:ietf:params:xml:ns:xmpp-tls'
    interfaces: ClassVar[Set[str]] = set()

    def exception(self, e: Exception) -> None:
        log.exception('Error handling {%s}%s stanza',
                      self.namespace, self.name)
        if isinstance(e, InvalidCABundle):
            raise e


class Failure(StanzaBase):
    """

    .. code-block:: xml

        <failure xmlns='urn:ietf:params:xml:ns:xmpp-tls'/>

    """
    name = 'failure'
    namespace = 'urn:ietf:params:xml:ns:xmpp-tls'
    interfaces: ClassVar[Set[str]] = set()



#!/usr/bin/env python3

import sys
import logging
import unittest

from argparse import ArgumentParser
from importlib import import_module
from pathlib import Path


def run_tests(filenames=None, debug=False):
    """
    Find and run all tests in the tests/ directory.

    Excludes live tests (tests/live_*).
    """
    if sys.version_info < (3, 8):
        raise ValueError('Your python version is too old to run these tests')
    if not filenames:
        filenames = [i for i in Path('itests').glob('test_*')]
    else:
        filenames = [Path(i) for i in filenames]

    modules = ['.'.join(test.parts[:-1] + (test.stem,)) for test in filenames]

    suites = []
    for filename in modules:
        module = import_module(filename)
        suites.append(module.suite)

    tests = unittest.TestSuite(suites)
    runner = unittest.TextTestRunner(verbosity=2)

    if debug:
        logging.basicConfig(level='DEBUG')
    else:
        # Disable logging output
        logging.basicConfig(level=100)
        logging.disable(100)

    result = runner.run(tests)
    return result


if __name__ == '__main__':
    parser = ArgumentParser(description='Run unit tests.')
    parser.add_argument('tests', metavar='TEST', nargs='*', help='list of tests to run, or nothing to run them all')
    parser.add_argument('-d', '--debug', action='store_true', dest='debug', default=False, help='enable debug output')
    args = parser.parse_args()

    result = run_tests(args.tests, args.debug)
    print("<tests %s ran='%s' errors='%s' fails='%s' success='%s'/>" % (
        "xmlns='http//andyet.net/protocol/tests'",
        result.testsRun, len(result.errors),
        len(result.failures), result.wasSuccessful()))

    sys.exit(not result.wasSuccessful())



# Slixmpp: The Slick XMPP Library
# Copyright (C) 2011 Nathanael C. Fritz, Lance J.T. Stout
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.

import logging

from asyncio import Future
from typing import Optional
from slixmpp.plugins.base import BasePlugin
from slixmpp.plugins.xep_0108 import stanza, UserActivity


log = logging.getLogger(__name__)


class XEP_0108(BasePlugin):

    """
    XEP-0108: User Activity
    """

    name = 'xep_0108'
    description = 'XEP-0108: User Activity'
    dependencies = {'xep_0163'}
    stanza = stanza

    def plugin_end(self):
        self.xmpp['xep_0030'].del_feature(feature=UserActivity.namespace)
        self.xmpp['xep_0163'].remove_interest(UserActivity.namespace)

    def session_bind(self, jid):
        self.xmpp['xep_0163'].register_pep('user_activity', UserActivity)

    def publish_activity(self, general: str, specific: Optional[str] = None,
                         text: Optional[str] = None, **pubsubkwargs) -> Future:
        """
        Publish the user's current activity.

        :param general: The required general category of the activity.
        :param specific: Optional specific activity being done as part
                         of the general category.
        :param text: Optional natural-language description or reason
                     for the activity.
        """
        activity = UserActivity()
        activity['value'] = (general, specific)
        activity['text'] = text
        return self.xmpp['xep_0163'].publish(
            activity,
            node=UserActivity.namespace,
            **pubsubkwargs
        )

    def stop(self, **pubsubkwargs) -> Future:
        """
        Clear existing user activity information to stop notifications.
        """
        activity = UserActivity()
        return self.xmpp['xep_0163'].publish(
            activity,
            node=UserActivity.namespace,
            **pubsubkwargs
        )



import unittest
from slixmpp import Message
from slixmpp.test import SlixTest
from slixmpp.plugins.xep_0424 import stanza


class TestRetraction(SlixTest):

    def setUp(self):
        stanza.register_plugins()

    def testRetract(self):
        message = Message()
        message['retract']['id'] = 'some-id'

        self.check(message, """
<message>
  <retract xmlns="urn:xmpp:message-retract:1" id="some-id"/>
</message>
        """, use_values=False)

    def testRetracted(self):
        message = Message()
        message['retracted']['stamp'] = '2019-09-20T23:09:32Z'
        message['retracted']['id'] = 'originid'

        self.check(message, """
<message>
  <retracted stamp="2019-09-20T23:09:32Z"
             xmlns="urn:xmpp:message-retract:1"
             id="originid" /> 
</message>
        """)


suite = unittest.TestLoader().loadTestsFromTestCase(TestRetraction)



#!/usr/bin/env python3

# Slixmpp: The Slick XMPP Library
# Copyright (C) 2010  Nathanael C. Fritz
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.

import logging
from getpass import getpass
from argparse import ArgumentParser

import asyncio
import slixmpp
from slixmpp.exceptions import IqError, IqTimeout


class RegisterBot(slixmpp.ClientXMPP):

    """
    A basic bot that will attempt to register an account
    with an XMPP server.

    NOTE: This follows the very basic registration workflow
          from XEP-0077. More advanced server registration
          workflows will need to check for data forms, etc.
    """

    def __init__(self, jid, password):
        slixmpp.ClientXMPP.__init__(self, jid, password)

        # The session_start event will be triggered when
        # the bot establishes its connection with the server
        # and the XML streams are ready for use. We want to
        # listen for this event so that we we can initialize
        # our roster.
        self.add_event_handler("session_start", self.start)

        # The register event provides an Iq result stanza with
        # a registration form from the server. This may include
        # the basic registration fields, a data form, an
        # out-of-band URL, or any combination. For more advanced
        # cases, you will need to examine the fields provided
        # and respond accordingly. Slixmpp provides plugins
        # for data forms and OOB links that will make that easier.
        self.add_event_handler("register", self.register)

    async def start(self, event):
        """
        Process the session_start event.

        Typical actions for the session_start event are
        requesting the roster and broadcasting an initial
        presence stanza.

        Arguments:
            event -- An empty dictionary. The session_start
                     event does not provide any additional
                     data.
        """
        self.send_presence()
        await self.get_roster()

        # We're only concerned about registering, so nothing more to do here.
        self.disconnect()

    async def register(self, iq):
        """
        Fill out and submit a registration form.

        The form may be composed of basic registration fields, a data form,
        an out-of-band link, or any combination thereof. Data forms and OOB
        links can be checked for as so:

        if iq.match('iq/register/form'):
            # do stuff with data form
            # iq['register']['form']['fields']
        if iq.match('iq/register/oob'):
            # do stuff with OOB URL
            # iq['register']['oob']['url']

        To get the list of basic registration fields, you can use:
            iq['register']['fields']
        """
        resp = self.Iq()
        resp['type'] = 'set'
        resp['register']['username'] = self.boundjid.user
        resp['register']['password'] = self.password

        try:
            await resp.send()
            logging.info("Account created for %s!" % self.boundjid)
        except IqError as e:
            logging.error("Could not register account: %s" %
                    e.iq['error']['text'])
            self.disconnect()
        except IqTimeout:
            logging.error("No response from server.")
            self.disconnect()


if __name__ == '__main__':
    # Setup the command line arguments.
    parser = ArgumentParser()

    # Output verbosity options.
    parser.add_argument("-q", "--quiet", help="set logging to ERROR",
                        action="store_const", dest="loglevel",
                        const=logging.ERROR, default=logging.INFO)
    parser.add_argument("-d", "--debug", help="set logging to DEBUG",
                        action="store_const", dest="loglevel",
                        const=logging.DEBUG, default=logging.INFO)

    # JID and password options.
    parser.add_argument("-j", "--jid", dest="jid",
                        help="JID to use")
    parser.add_argument("-p", "--password", dest="password",
                        help="password to use")

    args = parser.parse_args()

    # Setup logging.
    logging.basicConfig(level=args.loglevel,
                        format='%(levelname)-8s %(message)s')

    if args.jid is None:
        args.jid = input("Username: ")
    if args.password is None:
        args.password = getpass("Password: ")

    # Setup the RegisterBot and register plugins. Note that while plugins may
    # have interdependencies, the order in which you register them does
    # not matter.
    xmpp = RegisterBot(args.jid, args.password)
    xmpp.register_plugin('xep_0030') # Service Discovery
    xmpp.register_plugin('xep_0004') # Data forms
    xmpp.register_plugin('xep_0066') # Out-of-band Data
    xmpp.register_plugin('xep_0077') # In-band Registration

    # Some servers don't advertise support for inband registration, even
    # though they allow it. If this applies to your server, use:
    xmpp['xep_0077'].force_registration = True

    # Connect to the XMPP server and start processing XMPP stanzas.
    xmpp.connect()
    asyncio.get_event_loop().run_forever()




# Slixmpp: The Slick XMPP Library
# Copyright (C) 2012 Nathanael C. Fritz, Lance J.T. Stout
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.
import logging
import ssl

from slixmpp.stanza import StreamFeatures, Iq
from slixmpp.xmlstream import register_stanza_plugin, JID
from slixmpp.xmlstream.handler import CoroutineCallback
from slixmpp.xmlstream.matcher import StanzaPath
from slixmpp.plugins import BasePlugin
from slixmpp.plugins.xep_0077 import stanza, Register, RegisterFeature


log = logging.getLogger(__name__)


class XEP_0077(BasePlugin):

    """
    XEP-0077: In-Band Registration

    Events:

    ::

        user_register           -- After succesful validation and add to the user store
                                   in api["user_validate"]
        user_unregister         -- After succesful user removal in api["user_remove"]

    Config:

    ::

        form_fields are only form_instructions are only used for component registration
        in case api["make_registration_form"] is not overriden.

    API:

    ::

        user_get(jid, node, ifrom, iq)
            Returns a dict-like object containing `form_fields` for this user or None
        user_remove(jid, node, ifrom, iq)
            Removes a user or raise KeyError in case the user is not found in the user store
        make_registration_form(self, jid, node, ifrom, iq)
            Returns an iq reply to a registration form request, pre-filled and with
            <registered/> in case the requesting entity is already registered to us
        user_validate((self, jid, node, ifrom, registration)
            Add the user to the user store or raise ValueError(msg) if any problem is encountered
            msg is sent back to the XMPP client as an error message.
    """

    name = 'xep_0077'
    description = 'XEP-0077: In-Band Registration'
    dependencies = {'xep_0004', 'xep_0066'}
    stanza = stanza
    default_config = {
        'create_account': True,
        'force_registration': False,
        'order': 50,
        "form_fields": {"username", "password"},
        "form_instructions": "Enter your credentials",
    }

    def plugin_init(self):
        register_stanza_plugin(StreamFeatures, RegisterFeature)
        register_stanza_plugin(Iq, Register)

        if self.xmpp.is_component:
            self.xmpp["xep_0030"].add_feature("jabber:iq:register")
            self.xmpp.register_handler(
                CoroutineCallback(
                    "registration",
                    StanzaPath("/iq/register"),
                    self._handle_registration,
                )
            )
            self._user_store = {}
            self.api.register(self._user_get, "user_get")
            self.api.register(self._user_remove, "user_remove")
            self.api.register(self._make_registration_form, "make_registration_form")
            self.api.register(self._user_validate, "user_validate")
        else:
            self.xmpp.register_feature(
                "register",
                self._handle_register_feature,
                restart=False,
                order=self.order,
            )

        register_stanza_plugin(Register, self.xmpp['xep_0004'].stanza.Form)
        register_stanza_plugin(Register, self.xmpp['xep_0066'].stanza.OOB)

        self.xmpp.add_event_handler('connected', self._force_registration)

    def plugin_end(self):
        if not self.xmpp.is_component:
            self.xmpp.unregister_feature('register', self.order)

    def _user_get(self, jid, node, ifrom, iq):
        return self._user_store.get(iq["from"].bare)

    def _user_remove(self, jid, node, ifrom, iq):
        return self._user_store.pop(iq["from"].bare)

    async def _make_registration_form(self, jid, node, ifrom, iq: Iq):
        reg = iq["register"]
        user = await self.api["user_get"](None, None, iq['from'], iq)

        if user is None:
            user = {}
        else:
            reg["registered"] = True

        reg["instructions"] = self.form_instructions

        for field in self.form_fields:
            data = user.get(field, "")
            if data:
                reg[field] = data
            else:
                # Add a blank field
                reg.add_field(field)

        reply = iq.reply()
        reply.set_payload(reg.xml)
        return reply

    def _user_validate(self, jid, node, ifrom, registration):
        self._user_store[ifrom.bare] = {key: registration[key] for key in self.form_fields}

    async def _handle_registration(self, iq: Iq):
        if iq["type"] == "get":
            await self._send_form(iq)
        elif iq["type"] == "set":
            if iq["register"]["remove"]:
                try:
                    await self.api["user_remove"](None, None, iq["from"], iq)
                except KeyError:
                    _send_error(
                        iq,
                        "404",
                        "cancel",
                        "item-not-found",
                        "User not found",
                    )
                else:
                    reply = iq.reply()
                    reply.send()
                    self.xmpp.event("user_unregister", iq)
                return


            for field in self.form_fields:
                if not iq["register"][field]:
                    # Incomplete Registration
                    _send_error(
                        iq,
                        "406",
                        "modify",
                        "not-acceptable",
                        "Please fill in all fields.",
                    )
                    return

            try:
                await self.api["user_validate"](None, None, iq["from"], iq["register"])
            except ValueError as e:
                _send_error(
                    iq,
                    "406",
                    "modify",
                    "not-acceptable",
                    e.args,
                )
            else:
                reply = iq.reply()
                reply.send()
                self.xmpp.event("user_register", iq)

    async def _send_form(self, iq):
        reply = await self.api["make_registration_form"](None, None, iq["from"], iq)
        reply.send()

    def _force_registration(self, event):
        if self.force_registration:
            self.xmpp.add_filter('in', self._force_stream_feature)

    def _force_stream_feature(self, stanza):
        if isinstance(stanza, StreamFeatures):
            if not self.xmpp.disable_starttls:
                if 'starttls' not in self.xmpp.features:
                    return stanza
                elif not isinstance(self.xmpp.socket, ssl.SSLSocket):
                    return stanza
            if 'mechanisms' not in self.xmpp.features:
                log.debug('Forced adding in-band registration stream feature')
                stanza.enable('register')
                self.xmpp.del_filter('in', self._force_stream_feature)
        return stanza

    async def _handle_register_feature(self, features):
        if 'mechanisms' in self.xmpp.features:
            # We have already logged in with an account
            return False

        if self.create_account and self.xmpp.event_handled('register'):
            form = await self.get_registration()
            await self.xmpp.event_async('register', form)
            return True
        return False

    def get_registration(self, jid=None, ifrom=None,
                         timeout=None, callback=None):
        iq = self.xmpp.Iq()
        iq['type'] = 'get'
        iq['to'] = jid
        iq['from'] = ifrom
        iq.enable('register')
        return iq.send(timeout=timeout, callback=callback)

    def cancel_registration(self, jid=None, ifrom=None,
                            timeout=None, callback=None):
        iq = self.xmpp.Iq()
        iq['type'] = 'set'
        iq['to'] = jid
        iq['from'] = ifrom
        iq['register']['remove'] = True
        return iq.send(timeout=timeout, callback=callback)

    def change_password(self, password, jid=None, ifrom=None,
                        timeout=None, callback=None):
        iq = self.xmpp.Iq()
        iq['type'] = 'set'
        iq['to'] = jid
        iq['from'] = ifrom
        if self.xmpp.is_component:
            ifrom = JID(ifrom)
            iq['register']['username'] = ifrom.user
        else:
            iq['register']['username'] = self.xmpp.boundjid.user
        iq['register']['password'] = password
        return iq.send(timeout=timeout, callback=callback)

def _send_error(iq, code, error_type, name, text=""):
    # It would be nice to raise XMPPError but the iq payload
    # should include the register info
    reply = iq.reply()
    reply.set_payload(iq["register"].xml)
    reply.error()
    reply["error"]["code"] = code
    reply["error"]["type"] = error_type
    reply["error"]["condition"] = name
    reply["error"]["text"] = text
    reply.send()




# Slixmpp: The Slick XMPP Library
# Copyright (C) 2017 Emmanuel Gil Peyrot <linkmauve@linkmauve.fr>
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.
from slixmpp.xmlstream import ElementBase, register_stanza_plugin, ET


class Markup(ElementBase):
    namespace = 'urn:xmpp:markup:0'
    name = 'markup'
    plugin_attrib = 'markup'


class _FirstLevel(ElementBase):
    namespace = 'urn:xmpp:markup:0'
    interfaces = {'start', 'end'}

    def get_start(self):
        return int(self._get_attr('start'))

    def set_start(self, value):
        self._set_attr('start', '%d' % value)

    def get_end(self):
        return int(self._get_attr('end'))

    def set_end(self, value):
        self._set_attr('end', '%d' % value)

class Span(_FirstLevel):
    name = 'span'
    plugin_attrib = 'span'
    plugin_multi_attrib = 'spans'
    interfaces = {'start', 'end', 'types'}

    def get_types(self):
        types = []
        if self.xml.find('{urn:xmpp:markup:0}emphasis') is not None:
            types.append('emphasis')
        if self.xml.find('{urn:xmpp:markup:0}code') is not None:
            types.append('code')
        if self.xml.find('{urn:xmpp:markup:0}deleted') is not None:
            types.append('deleted')
        return types

    def set_types(self, value):
        del self['types']
        for type_ in value:
            if type_ == 'emphasis':
                self.xml.append(ET.Element('{urn:xmpp:markup:0}emphasis'))
            elif type_ == 'code':
                self.xml.append(ET.Element('{urn:xmpp:markup:0}code'))
            elif type_ == 'deleted':
                self.xml.append(ET.Element('{urn:xmpp:markup:0}deleted'))

    def det_types(self):
        for child in self.xml:
            self.xml.remove(child)


class _SpanType(ElementBase):
    namespace = 'urn:xmpp:markup:0'


class EmphasisType(_SpanType):
    name = 'emphasis'
    plugin_attrib = 'emphasis'


class CodeType(_SpanType):
    name = 'code'
    plugin_attrib = 'code'


class DeletedType(_SpanType):
    name = 'deleted'
    plugin_attrib = 'deleted'


class BlockCode(_FirstLevel):
    name = 'bcode'
    plugin_attrib = 'bcode'
    plugin_multi_attrib = 'bcodes'


class List(_FirstLevel):
    name = 'list'
    plugin_attrib = 'list'
    plugin_multi_attrib = 'lists'
    interfaces = {'start', 'end', 'li'}


class Li(ElementBase):
    namespace = 'urn:xmpp:markup:0'
    name = 'li'
    plugin_attrib = 'li'
    plugin_multi_attrib = 'lis'
    interfaces = {'start'}

    def get_start(self):
        return int(self._get_attr('start'))

    def set_start(self, value):
        self._set_attr('start', '%d' % value)


class BlockQuote(_FirstLevel):
    name = 'bquote'
    plugin_attrib = 'bquote'
    plugin_multi_attrib = 'bquotes'

register_stanza_plugin(Markup, Span, iterable=True)
register_stanza_plugin(Markup, BlockCode, iterable=True)
register_stanza_plugin(Markup, List, iterable=True)
register_stanza_plugin(Markup, BlockQuote, iterable=True)
register_stanza_plugin(Span, EmphasisType)
register_stanza_plugin(Span, CodeType)
register_stanza_plugin(Span, DeletedType)
register_stanza_plugin(List, Li, iterable=True)



"""
    Slixmpp: The Slick XMPP Library
    Copyright (C) 2017 Emmanuel Gil Peyrot
    This file is part of Slixmpp.

    See the file LICENSE for copying permission.
"""

import unittest
from slixmpp import Iq
from slixmpp.test import SlixTest
from slixmpp.plugins.xep_0300 import Hash
from slixmpp.xmlstream import register_stanza_plugin


class TestHash(SlixTest):

    def setUp(self):
        register_stanza_plugin(Iq, Hash)

    def testSimpleElement(self):
        """Test that the element is created correctly."""
        iq = Iq()
        iq['type'] = 'set'
        iq['hash']['algo'] = 'sha-256'
        iq['hash']['value'] = 'EQgS9n+h4fARf289cCQcGkKnsHcRqTwkd8xRbZBC+ds='

        self.check(iq, """
          <iq type="set">
            <hash xmlns="urn:xmpp:hashes:2" algo="sha-256">EQgS9n+h4fARf289cCQcGkKnsHcRqTwkd8xRbZBC+ds=</hash>
          </iq>
        """)

    def testInvalidAlgo(self):
        """Test that invalid algos raise an exception."""
        iq = Iq()
        iq['type'] = 'set'
        try:
            iq['hash']['algo'] = 'coucou'
        except ValueError:
            pass
        else:
            raise self.failureException

    #def testDisabledAlgo(self):
    #    """Test that disabled algos aren’t used."""
    #    iq = Iq()
    #    iq['type'] = 'set'
    #    try:
    #        iq['hash']['algo'] = 'sha-1'
    #    except ValueError:
    #        pass
    #    else:
    #        raise self.failureException


suite = unittest.TestLoader().loadTestsFromTestCase(TestHash)




# Slixmpp: The Slick XMPP Library
# Copyright (C) 2012 Nathanael C. Fritz, Lance J.T. Stout
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.
import logging

from slixmpp.plugins import BasePlugin
from slixmpp.xmlstream import register_stanza_plugin
from slixmpp.plugins.xep_0221 import stanza, Media, URI
from slixmpp.plugins.xep_0004 import FormField


log = logging.getLogger(__name__)


class XEP_0221(BasePlugin):
    """
    XEP-0221: Data Forms Media Element

    In certain implementations of Data Forms (XEP-0004), it can be
    helpful to include media data such as small images. One example is
    CAPTCHA Forms (XEP-0158). This plugin implements a method for
    including media data in a data form.

    Typical use pattern:

    .. code-block:: python

        self.register_plugin('xep_0221')
        self['xep_0050'].add_command(node="showimage",
                                        name="Show my image",
                                        handler=self.form_handler)

        def form_handler(self,iq,session):
            image_url="https://xmpp.org/images/logos/xmpp-logo.svg"
            form=self['xep_0004'].make_form('result','My Image')
            form.addField(var='myimage', ftype='text-single', label='My Image', value=image_url)
            form.field['myimage']['media'].add_uri(value=image_url, itype="image/svg")
            session['payload']=form
            return session
    """


    name = 'xep_0221'
    description = 'XEP-0221: Data Forms Media Element'
    dependencies = {'xep_0004'}

    def plugin_init(self):
        register_stanza_plugin(FormField, Media)



import asyncio
import unittest
from slixmpp import JID
from slixmpp.test.integration import SlixIntegration
from hashlib import sha1


class TestVcardAvatar(SlixIntegration):
    async def asyncSetUp(self):
        await super().asyncSetUp()
        self.add_client(
            self.envjid('CI_ACCOUNT1'),
            self.envstr('CI_ACCOUNT1_PASSWORD'),
        )
        self.register_plugins(['xep_0153'])
        self.data = b'coucou coucou'
        self.hashed_data = sha1(self.data).hexdigest()
        await self.connect_clients()

    async def _clear_avatar(self):
        """Utility for purging remote state"""
        await self.clients[0]['xep_0153'].set_avatar(avatar=b'')

    async def test_set_avatar(self):
        """Check we can set and get a PEP avatar and metadata"""
        await self._clear_avatar()

        event = self.clients[0].wait_until('vcard_avatar_update')
        update = self.clients[0]['xep_0153'].set_avatar(
            avatar=self.data
        )
        result = await asyncio.gather(
            event,
            update,
        )
        presence = result[0]
        hash = presence['vcard_temp_update']['photo']
        self.assertEqual(hash, self.hashed_data)

        iq = await self.clients[0]['xep_0054'].get_vcard(
            JID(self.clients[0].boundjid.bare)
        )
        photo = iq['vcard_temp']['PHOTO']['BINVAL']
        self.assertEqual(photo, self.data)

        await self._clear_avatar()


suite = unittest.TestLoader().loadTestsFromTestCase(TestVcardAvatar)



import unittest
from slixmpp import Presence, JID
from slixmpp.test import SlixTest
from slixmpp.plugins.xep_0403 import stanza


class TestMIXPresenceStanza(SlixTest):

    def setUp(self):
        stanza.register_plugins()

    def testMIXPresence(self):
        """Test that data is converted to base64"""
        pres = Presence()
        pres['show'] = 'dnd'
        pres['status'] = 'Hey there!'
        pres['mix']['jid'] = JID('toto@example.com')
        pres['mix']['nick'] = 'Toto toto'

        self.check(pres, """
          <presence>
             <show>dnd</show>
             <status>Hey there!</status>
             <mix xmlns="urn:xmpp:mix:presence:0">
                 <jid>toto@example.com</jid>
                 <nick>Toto toto</nick>
             </mix>
          </presence>
        """)


suite = unittest.TestLoader().loadTestsFromTestCase(TestMIXPresenceStanza)




# Slixmpp: The Slick XMPP Library
# Copyright (C) 2013 Nathanael C. Fritz, Lance J.T. Stout
# This file is part of slixmpp.
# See the file LICENSE for copying permission.
from slixmpp.xmlstream import ElementBase, ET


class GoogleAuth(ElementBase):
    name = 'auth'
    namespace = 'http://www.google.com/talk/protocol/auth'
    plugin_attrib = 'google'
    interfaces = {'client_uses_full_bind_result', 'service'}

    discovery_attr= '{%s}client-uses-full-bind-result' % namespace
    service_attr= '{%s}service' % namespace

    def setup(self, xml):
        """Don't create XML for the plugin."""
        self.xml = ET.Element('')

    def get_client_uses_full_bind_result(self):
        return self.parent()._get_attr(self.discovery_attr) == 'true'

    def set_client_uses_full_bind_result(self, value):
        if value in (True, 'true'):
            self.parent()._set_attr(self.discovery_attr, 'true')
        else:
            self.parent()._del_attr(self.discovery_attr)

    def del_client_uses_full_bind_result(self):
        self.parent()._del_attr(self.discovery_attr)

    def get_service(self):
        return self.parent()._get_attr(self.service_attr, '')

    def set_service(self, value):
        if value:
            self.parent()._set_attr(self.service_attr, value)
        else:
            self.parent()._del_attr(self.service_attr)

    def del_service(self):
        self.parent()._del_attr(self.service_attr)
